## auxiliary functions --------------------------------------------------
## Install the required packages manually -------------------------------

library(dplyr)
library(ggplot2)
library(readr)
library(tidyr)
library(lubridate)
library(alfred)
library(leaps)
library(glmnet)
library(foreach)
library(doParallel)
library(doSNOW)
library(RColorBrewer)
library(formula.tools)
library(readxl)
library(quantreg)
library(WRTDStidal)
library(grid)
library(gridExtra)
library(L0Learn)
library(tibbletime)
library(ggsci)
library(stargazer)
library(lmtest)
library(sandwich)
library(cowplot)
library(HDeconometrics) ## https://github.com/gabrielrvsc/HDeconometrics
library(mutoss) ## https://github.com/kornl/mutoss
library(multtest) ## https://www.bioconductor.org/packages/release/bioc/html/multtest.html

pick <- function(condition){
  function(d) d %>% filter_(condition)
}


# recession dates  ----------------------------------------

cepr.min <- c("2008-04-01", "2011-10-01", "2020-01-01") 
cepr.max <- c("2009-06-01","2013-03-01", "2020-07-01")
cepr.min <- as.Date(cepr.min)
cepr.max <- as.Date(cepr.max)

nber.min <- c("1981-07-01", "1990-07-01", "2001-03-01", "2008-01-01")
nber.max <- c("1982-12-01", "1991-04-01", "2001-12-01", "2009-07-01")
nber.min <- as.Date(nber.min)
nber.max <- as.Date(nber.max)

brexit <- as.Date("2016-06-23")


## Load sentiment indicators built from DNA for Update 2020
get_sentiment <- function(country, folder = folder.sent, 
                          period="before2019", 
                          ttense=NULL, 
                          OSS=FALSE){
  
  if (is.null(ttense)){ttense = c("PaPrFuNaN", "Pa", "Pr", "Fu", "NaN")}
  
  ## PERIODS
  if (period == "before2019"){start=as.Date("1995-01-01");end=as.Date("2019-09-01")}
  if (period == "2019-2020"){ start=as.Date("2019-09-02");end=as.Date("2020-03-08")}
  if (period == "2020"){ start=as.Date("2020-03-09");end=as.Date("2020-09-30")}
  if (period == "2020_Dec"){ start=as.Date("2020-10-01");end=as.Date("2020-12-31")}
  
  # loads the sentiment data from folders
  econ.dirs      <- list.dirs(paste0(folder,"Economy"), full.names  = TRUE, recursive = FALSE)  
  unemp.dirs     <- list.dirs(paste0(folder,"Unemployment"), full.names  = TRUE, recursive = FALSE)  
  #monpolEU.dirs   <- list.dirs(paste0(folder,"MonPol"), full.names  = TRUE, recursive = FALSE)  
  monpolEU.dirs  <- list.dirs(paste0(folder,"MonPol_EU"), full.names  = TRUE, recursive = FALSE)  
  indp.dirs      <- list.dirs(paste0(folder,"IndP"), full.names  = TRUE, recursive = FALSE)  
  infl.dirs      <- list.dirs(paste0(folder,"Inflation"), full.names  = TRUE, recursive = FALSE)  
  fins.dirs      <- list.dirs(paste0(folder,"FinSector"), full.names  = TRUE, recursive = FALSE)  
  
  all.dirs  <- c(econ.dirs,  unemp.dirs, monpolEU.dirs, indp.dirs, infl.dirs,fins.dirs)
  
  ## Select only sentiment built with CAS2Dict (annotated dictionary from Nov 2019) and the tense of interest and the Country of interest
  all.dirs <- all.dirs[grepl(ttense, all.dirs) & grepl(country, all.dirs) & grepl("CAS2Dict", all.dirs)] 
  
  ## OSS
  if(OSS == FALSE){
    all.dirs <- all.dirs[!grepl("OSS", all.dirs)]
  } else {
    all.dirs <- all.dirs[grepl("OSS", all.dirs)]
  }
  
  ## PERIOD of interest 
  if (period == "before2019"){all.dirs <- all.dirs[!grepl("Update", all.dirs)]}
  if (period == "2019-2020"){all.dirs <- all.dirs[grepl("Update_2019_2020", all.dirs)]}
  if (period == "2020"){all.dirs <- all.dirs[grepl("Update_2020", all.dirs) & !grepl("Update_2020_Dec", all.dirs)]}
  if (period == "2020_Dec"){all.dirs <- all.dirs[grepl("Update_2020_Dec", all.dirs)]}
  
  
  df.sent    <- data.frame(realtime_period = seq(start, end,by = "days"))
  df.extract <- df.sent
  
  for (i in 1:length(all.dirs)){
    dir = all.dirs[i]
    print(dir)
    setwd(dir)
    
    tense_all <- strsplit(dir, "/")
    
    gsub("Update.*", "", tense_all[[1]][length(tense_all[[1]])])
    
    tense <- gsub("_Update.*", "", tense_all[[1]][length(tense_all[[1]])]) %>% # remove everything after Update
      strsplit("_") %>% unlist 
    tense <- tense[-which(tense == "CAS2Dict")]
    average          <- read.csv("average_sentiment.csv", header = FALSE)
    cumulative       <- read.csv("cumulative_sentiment.csv", header = FALSE)
    grouping_results <- readr::read_delim("grouping_results.csv", 
                                          ";", 
                                          escape_double = FALSE, 
                                          col_types = cols(`Grouped Dates` = col_date(format = "(%Y, %m, %d)")), 
                                          trim_ws = TRUE) %>% data.frame
    
    n_extract <- strsplit(grouping_results$Articles.Indexes,",")
    n_extract <- sapply(n_extract, function(x) sum(x != "[]"))  
    
    ## Sentiment
    data = data.frame(grouping_results$Grouped.Dates, 
                      average,
                      cumulative) 
    
    # if (folder == "/DATA/COVID19/Saved_Simulations_COVID19/") {
    #   colnames(data)  <- c("realtime_period", paste0("av_", tense[2], "_", tense[4]), paste0("cum_", tense[2], "_", tense[4]))
    # }
    if (folder == folder.sent) {
      if (length(tense) == 4) colnames(data)  <- c("realtime_period", paste0("av_", tense[2], "_", tense[4]), paste0("cum_", tense[2], "_", tense[4]))
      if (length(tense) == 5)  colnames(data) <- c("realtime_period", paste0("av_", tense[2], "_", tense[4],"_", tense[5]), paste0("cum_", tense[2], "_", tense[4], "_", tense[5]))
      if (length(tense) == 6)  colnames(data) <- c("realtime_period", paste0("av_", tense[2], "_", tense[4],"_", tense[5],"_", tense[6]), paste0("cum_", tense[2], "_", tense[4], "_", tense[5], "_", tense[6]))
    }
    df.sent <- left_join(df.sent, data, by = "realtime_period")
    if (i == length(all.dirs)) print(dim(df.sent))
    
    
    ## N-extractions
    data <- data.frame(grouping_results$Grouped.Dates, 
                       n_extract)
    
    # if (folder == "/DATA/COVID19/Saved_Simulations_COVID19/") {
    #   colnames(data) <- c("realtime_period", paste0("n_", tense[2], "_", tense[4]))
    # }
    if (folder == folder.sent) {
      if (length(tense) == 4) colnames(data) <- c("realtime_period", paste0("n_", tense[2], "_", tense[4]))
      if (length(tense) == 5) colnames(data) <- c("realtime_period", paste0("n_", tense[2], "_", tense[4], "_", tense[5]))
      if (length(tense) == 6) colnames(data) <- c("realtime_period", paste0("n_", tense[2], "_", tense[4], "_", tense[5], "_", tense[6]))
    }
    
    df.extract <- left_join(df.extract, data, by = "realtime_period")
  }
  
  names(df.sent)    <- tolower(names(df.sent))
  names(df.extract) <- tolower(names(df.extract))
  if(OSS == TRUE)
  {
    names(df.sent) <- stringr::str_replace(names(df.sent), "_oss","")
    names(df.extract) <- stringr::str_replace(names(df.extract), "_oss","")
  }
  
  
  
  #   df.sent[is.na(df.sent[,-1]),-1] <- 0
  df.sent <- arrange(df.sent, realtime_period)
  df.extract <- df.extract %>% arrange(realtime_period)
  
  return(list(df.sent, df.extract))
}



## ETL on Sentiment Indicators
ETL_sentiment <- function(x, country, measures = c("av", "cum", "n"), tense="paprfunan") {
  
  ###########
  ## INPUT ##
  ###########
  
  # x : output of get_sentiment()
  # country : one of "DE", "ES", "FR", "IT , "UK", "US"
  # measure : "av" (average) or "cum" (cumulative) sentiment measure
  # tense : a vector. Default "PaPrFuNaN". Write it all lower case. Example c("pa", "pr", "fu", "paprfunan")
  
  x_sent <-x[[1]] %>%
    select(-ends_with("oss")) %>%
    select(realtime_period, ends_with(tense))
  
  x_n <-x[[2]] %>%
    select(-ends_with("oss")) %>%
    select(realtime_period, ends_with(tense))
  
  
  library(tidyr)
  if (sum(measures %in% c("av", "cum"))>0){
    out1 <- as_tibble(x_sent) %>%
      # select(realtime_period, starts_with("av_")) %>%
      gather(var, value, -realtime_period) %>%
      separate(var, sep="_", into=c("measure", "topic", "tense")) %>%
      filter(measure %in% measures) %>%
      mutate(country = country,
             value = as.numeric(value))
  } else {
    out1 <- NULL
  }
  
  if (sum(measures %in% c("n"))>0){
    out2 <- as_tibble(x_n) %>%
      gather(var, value, -realtime_period) %>%
      separate(var, sep="_", into=c("measure", "topic", "tense")) %>%
      filter(measure %in% measures) %>%
      mutate(country = country,
             value = as.numeric(value))
  } else {
    out2 <- NULL
  }
  
  out <- rbind(out1, out2)
  return(out)
  
}

## Smooth sentiment indicators
roll_sentiment <- function(sentiment.df, type = "ema", lambda = 0.07, roll_window = 60, fun = "mean"){
  
  # smooth daily sentiment series
  
  ###########
  ## INPUT ##
  ###########
  
  #    sentiment.df = sentiment in long format 
  #    type         = "ema" (exponential moving average) or "sma" (simple moving average)
  #    lambda       = parameter for EMA (between 0 and 1)
  #    roll_window  = parameter for SMA (unit in days)
  #    fun          = "mean" or "sum" 
  
  
  library(tibbletime)
  
  ma_fun <- function(x, lambda){
    weights = (1-lambda)^seq(length(x)-1, 0, -1)
    value = lambda  * sum(x * weights, na.rm = TRUE)
    return(value)
  }
  
  if (type == "sma") roll.mean   <- rollify(.f = mean, window = roll_window, unlist = TRUE)
  {
    if (fun == "sum") roll.mean   <- rollify(.f = sum, window = roll_window, unlist = TRUE) else roll.mean   <- rollify(.f = mean, window = roll_window, unlist = TRUE)
  }
  if (type == "ema") roll.mean   <- rollify(.f = ~ma_fun(.x, lambda), window = roll_window, unlist = TRUE)
  
  # sentiment   <- mutate_at(sentiment.df, 
  #                          vars(-c(realtime_period)), 
  #                          roll.mean)
  
  sentiment <- sentiment.df %>%
    group_by(measure, topic, tense, country) %>%
    mutate(value = roll.mean(value))
  
  return(sentiment)
}


roll_sentiment_US <- function(sentiment.df, type = "ema", lambda = 0.07, roll_window = 60, fun = "sum")
{
  # smooth daily sentiment series:
  #    type = "ema" or "sma"
  #    lambda = parameter for EMA (between 0 and 1)
  #    roll_window = = parameter for SMA (unit in days)
  
  
  library(tibbletime)
  
  ma_fun <- function(x, lambda){
    weights = (1-lambda)^seq(length(x)-1, 0, -1)
    value = lambda  * sum(x * weights, na.rm = TRUE)
    
    return(value)
  }
  mean1 <- function(x)
  {
    d = mean(x, na.rm = TRUE)
    return(d)
  }
  
  if (fun == "mean")
  {
    if (type == "sma") roll.mean   <- rollify(mean1, roll_window,  unlist = TRUE)
    if (type == "ema") roll.mean   <- rollify(.f = ~ma_fun(.x, lambda), window = 200, unlist = TRUE)
  }
  if (fun == "sum")
  {
    if (type == "sma") roll.mean   <- rollify(sum, roll_window, unlist = TRUE)
    ## this not changed ...
    if (type == "ema") roll.mean   <- rollify(.f = ~ma_fun(.x, lambda), window = 200, unlist = TRUE)
  }
  
  sentiment   <- mutate_at(sentiment.df, 
                           vars(-c(realtime_period)), 
                           roll.mean)
  return(sentiment)
}


## Guess releases for monthly and quaterly variables
guess_release_monthly <- function(Y)
{
  # learn from the last 100 rows the pattern of release
  
  YY <- tail(Y, 100)
  diff_dates = (YY$realtime_period - YY$ref_date)  
  rel_day   = table(diff_dates)
  rel_day   = names(rel_day)[which.max(rel_day)] %>% as.numeric
  
  
  first_vintage <- min(Y$realtime_period, na.rm = T) - 7
  
  Y <- Y %>%
    mutate(realtime_period = if_else(ref_date < first_vintage, 
                                     ref_date + rel_day,
                                     realtime_period),
           pub_lag = realtime_period - ref_date)
  return(Y)
}


## Guess releases for weekly variables
guess_release_weekly <- function(Y)
{
  # learn from the last 100 rows the pattern of release
  
  YY <- tail(Y, 100)
  diff_dates = (YY$realtime_period - YY$ref_date)
  rel_day   = table(diff_dates)
  rel_day   = names(rel_day)[which.max(rel_day)] %>% as.numeric
  
  first_vintage <- min(Y$realtime_period, na.rm = T) - 7
  
  Y <- Y %>%
    mutate(realtime_period = if_else(ref_date < first_vintage,
                                     ref_date + rel_day,
                                     realtime_period),
           pub_lag = realtime_period - ref_date)
  return(Y)
}


## Download data from ALFRED
get_macrodata <- function(country, 
                          where      = "alfred", 
                          start.date = "1997-01-01", 
                          end.date   = "2020-10-12", 
                          vintage    = 1,
                          save.it    = FALSE,
                          folder.fred = "./RData/FRED_EU/",
                          symb.list_all)
{
  # GET DATA FROM Alfred
  # country = "US", "IT", "FR", "UK","DE"
  # where   = "alfred" to download the data from or "disk"  if the data are already downloaded
  
  symb.list <- symb.list_all[symb.list_all$country == country, ]
  
  
  if (where == "alfred") {
    df = data.frame()
    
    
    for (i in 1:nrow(symb.list)) {
      symb <- symb.list$symb[i]
      
      if (save.it) {
        X <- get_alfred_series(symb, "Value", observation_start = start.date, observation_end   = end.date)
        readr::write_csv(x = X, path = paste0(folder.fred, symb,"_", country,".csv"), col_names = TRUE)
      }
      if(!save.it){X <- readr::read_csv(paste0(folder.fred, symb,"_", country,".csv"), col_types = list(col_date(), col_date(), col_double()))}
      
      
      if (symb.list$freq[i] == "m"){
        X <- X %>%
          rename(ref_date = date) %>%
          mutate(ref_date = ymd(paste0(year(ref_date), 
                                       "-", 
                                       month(ref_date), 
                                       "-", 
                                       days_in_month(ref_date)))) 
      }
      if (symb.list$freq[i] == "q"){
        X <- X %>%
          rename(ref_date = date) %>%
          mutate(ref_date = ref_date + months(2),
                 ref_date = ymd(paste0(year(ref_date), 
                                       "-", 
                                       month(ref_date), 
                                       "-", 
                                       days_in_month(ref_date)))) 
      } 
      if (symb.list$freq[i] %in% c("w", "d")){
        X <- X %>%
          rename(ref_date = date) 
      } 
      
      if (symb.list$transf[i] == "n") {
        if (vintage > 1) X1 <- 
            X %>% 
            arrange(ref_date) %>%
            group_by(ref_date) %>% 
            arrange(realtime_period) %>% 
            filter(row_number() == 1) %>% 
            ungroup 
        
        X <- X %>%
          arrange(ref_date) %>%
          group_by(ref_date) %>%
          arrange(realtime_period) %>%
          filter(row_number() == vintage) %>%
          ungroup %>%
          mutate(country = country,
                 Symbol  = symb, 
                 pub_lag = realtime_period - ref_date,
                 Event   = "data from fred")
        
        if (vintage > 1){
          X <- X %>%
            mutate(realtime_period = X1$realtime_period[1:nrow(X)]) %>%
            mutate(pub_lag = realtime_period - ref_date)
        }
      }
      
      if (symb.list$freq[i] == "q") mylag = 1
      if (symb.list$freq[i] == "m") mylag = 1
      if ((symb.list$freq[i] == "m") & (symb.list$Description[i] %in% c("CPI"))) mylag = 1
      if ((symb.list$freq[i] == "m") & (symb.list$Description[i] %in% c("INDPRO"))) mylag = 1
      if ((symb.list$freq[i] == "m") & (symb.list$Description[i] %in% c("UNEM"))) mylag = 1
      if (symb.list$freq[i] == "w") mylag = 4
      
      if (symb.list$transf[i] == "d"){
        if (vintage > 1) X1 <- X %>% arrange(ref_date) %>% group_by(realtime_period) %>% 
            mutate(Value  = Value - lag(Value, n = mylag, order_by = ref_date)) %>%
            group_by(ref_date) %>%arrange(realtime_period) %>% filter(row_number() == 1) %>% ungroup
        
        X <- X %>% 
          arrange(ref_date) %>%
          group_by(realtime_period) %>%
          mutate(Value  = Value - lag(Value, n = mylag, order_by = ref_date)) %>%
          group_by(ref_date) %>%
          arrange(realtime_period) %>%
          filter(row_number() == vintage) %>%
          ungroup %>%
          mutate(country = country,
                 Symbol = symb, 
                 pub_lag = realtime_period - ref_date,
                 Event = "data from fred")
        
        if (vintage > 1){
          X <- X %>%
            mutate(realtime_period = X1$realtime_period[1:nrow(X)]) %>%
            mutate(pub_lag = realtime_period - ref_date)
        }
      }
      
      if (symb.list$transf[i] == "ld"){
        if (vintage > 1) X1 <- 
            X %>% arrange(ref_date) %>% 
            group_by(realtime_period) %>% 
            mutate(Value  = 100 * ((Value / lag(Value, n = mylag, order_by = ref_date))^(12/mylag) - 1)) %>%
            group_by(ref_date) %>% 
            arrange(realtime_period) %>% 
            filter(row_number() == 1) %>% 
            ungroup
        
        if (symb.list$freq[i] == "q") exp1 = 4 else exp1 = 12
        
        X <- X %>% 
          arrange(ref_date) %>%
          group_by(realtime_period) %>%
          #mutate(Value  = 100 * ((Value / lag(Value, n = mylag, order_by = ref_date))^(exp1/mylag) - 1)) %>% ## Transformation used in FiGAS US (if used here, you get absurd values for 2020)
          mutate(Value  = 100 * ((Value - lag(Value, n = mylag, order_by = ref_date))/lag(Value, n = mylag, order_by = ref_date))) %>%
          group_by(ref_date) %>%
          arrange(realtime_period) %>%
          filter(row_number() == vintage) %>%
          ungroup %>%
          mutate(country = country,
                 Symbol = symb, 
                 pub_lag = realtime_period - ref_date,
                 Event = "data from fred")
        
        if (vintage > 1){
          X <- X %>%
            mutate(realtime_period = X1$realtime_period[1:nrow(X)]) %>%
            mutate(pub_lag = realtime_period - ref_date)
        }
      }
      
      if (symb.list$transf[i] == "ma3"){
        if (vintage > 1) X1 <- X %>% arrange(ref_date) %>% group_by(ref_date) %>% arrange(realtime_period) %>% filter(row_number() == 1) %>% ungroup 
        
        roll.mean3 <- rollify(mean, 3)
        
        X <- X %>%
          arrange(ref_date) %>%
          group_by(realtime_period) %>%
          mutate(Value  = roll.mean3(Value)) %>%
          group_by(ref_date) %>%
          arrange(realtime_period) %>%
          filter(row_number() == vintage) %>%
          ungroup %>%
          mutate(country = country,
                 Symbol  = symb, 
                 pub_lag = realtime_period - ref_date,
                 Event   = "data from fred")
        
        if (vintage > 1){
          X <- X %>%
            mutate(realtime_period = X1$realtime_period[1:nrow(X)]) %>%
            mutate(pub_lag = realtime_period - ref_date)
        }
      }
      
      # fix release date for montlhy variables
      
      if (symb.list$Description[i] %in% c("INDPRO","CPI","UNEM"))
      {
        X <- X %>%
          #rename(realtime_period_original = realtime_period) %>%
          mutate(realtime_period = if_else(as.double(pub_lag) > 50, 
                                           ref_date + lubridate::days(32), 
                                           realtime_period),
                 pub_lag = realtime_period - ref_date) 
      }
      
      
      # backcast some of the variables
      var.to.backcast <- c("CLVMNACSCAB1GQIT", "CLVMNACSCAB1GQFR", "CLVMNACSCAB1GQDE", "CLVMNACSCAB1GQES", "CLVMNACSCAB1GQUK", 
                           "ITAPROINDMISMEI", "FRAPROINDMISMEI", "DEUPROINDMISMEI", "ESPPROINDMISMEI", "GBRPROINDMISMEI", 
                           "ITACPIALLMINMEI", "FRACPIALLMINMEI", "DEUCPIALLMINMEI", "ESPCPIALLMINMEI", "GBRCPIALLMINMEI",
                           "LRHUTTTTITM156S","LRHUTTTTESM156S","LRHUTTTTFRM156S", "LRHUTTTTDEM156S","LRHUTTTTGBM156S")
      if (symb %in% var.to.backcast){X  <- guess_release_monthly(X)}      
      
      # create lags of high-frequency variables
      
      if (symb.list$freq[i] %in% c("m", "w")){
        lag_midas <- 3
        temp <- vector("list", lag_midas)
        for (i in 1:lag_midas) temp[[i]] = mutate(X, Value = lag(Value,i), Symbol = paste0(Symbol, "_",i))
        X = rbind(X, bind_rows(temp))
      }
      df <- rbind(df, X)
    }
  }  
  
  df.release   <- df %>% spread(Symbol, Value)
  df.pub_lag   <- df %>% select(-Value) %>% spread(Symbol, pub_lag)
  df.realtime  <- df.release %>% 
    select(-c(country, Event, ref_date, pub_lag)) %>% 
    group_by(realtime_period) %>% 
    summarize_all(mean, na.rm = TRUE) 
  
  # this creates a data frame for each day in the sample and the information
  # available on that day
  Xd <- data.frame(realtime_period = seq(as.Date(start.date), 
                                         as.Date(end.date),
                                         by = "days"))
  df.realtime <- left_join(Xd, df.realtime, by="realtime_period")
  
  # fill the NA with the latest value 
  df.realtime[df.realtime == 0] <- NA
  
  df.realtime <- df.realtime %>%
    mutate_at(vars(-c(realtime_period)), 
              funs(rel = as.character(cumsum(as.numeric(!is.na(.))))))
  
  
  f_temp <- function(df.temp) {
    N    <- sum(grepl("_rel", names(df.temp)))
    gvar <- names(df.temp)[grep("_rel", names(df.temp))]
    
    for (i in 1:N)
    {
      gvar_i <- gvar[i] 
      mvar_i <- substr(gvar_i, 1, nchar(gvar_i)-4)
      
      df.temp = 
        df.temp %>%
        group_by(!!ensym(gvar_i)) %>%
        mutate(!!mvar_i  := first(!!ensym(mvar_i))) %>%
        ungroup 
    }
    
    df.temp <- select(df.temp, -contains("_rel"))
    return(df.temp)
  }
  
  df.realtime <- f_temp(df.realtime)
  
  
  # plot histogram of release delay
  # df %>% 
  #   mutate(diff = realtime_period - ref_date) %>% 
  #   ggplot() + 
  #   geom_histogram(aes(diff)) + 
  #   theme_bw() + labs(x = NULL, y = NULL, title = "Release lag") + 
  #   facet_wrap(~ Symbol, scales = "free")
  
  return(list(df, df.release, df.pub_lag, df.realtime))
}  


get_stockprices <- function(country)
{
  library(tidyquant)
  tic <- data.frame(country = c("UK","DE","FR","ES","IT"), tick = c("^FTSE","^GDAXI","^FCHI","^IBEX","FTSEMIB.MI"))
  indexes <- tq_get(as.character(tic[tic$country == country.i, "tick"]), from= "1994-01-01")
  indexes <- indexes %>%
    select(date, adjusted) %>%
    rename(INDEX           = adjusted,
           realtime_period = date) %>%
    mutate(INDEX = 100*(log(INDEX) - lag(log(INDEX), 1)))
  return(indexes)
}



save_macrodata <- function(country, folder.raw = folder.raw, folder.save = folder.save)
{
  # uses the bloomberg calendar downloaded by Luca and creates a dataframe 
  # need to set the variables you are interested to create and the frequency in the 
  #   data frame called "info"
  #
  # reads the bloomberg files from .../mydata/DNA_AVRO/Sergio/Macro_Release_Dates/raw_data/
  # write the per-variable files to .../mydata/DNA_AVRO/Sergio/Macro_Release_Dates/country/<country>
  #
  #  countries     <- c("BE", "DE", "ES", "FR", "IE", "IT", "NL", "PT", "UK", "US")
  
  combine_files <- function(country, folder.raw){
    
    ll <- list.files(folder.raw)
    
    index   <- which(substr(ll, 1, 2) == country)
    data    <- lapply(paste0(folder.raw,ll[index]), function(x)  readr::read_csv(x))
    data    <- bind_rows(data)
    
    data <- data %>%
      rename(date      = `Date Time`,
             country   = X2, 
             ref_month = X6) %>%
      select(-c("X3", "X4")) %>%
      mutate(date   = substr(date, 1,10),
             date   = as.Date(date, "%m/%d/%Y"),
             Survey = as.numeric(gsub("%", "", Survey)),
             Actual = as.numeric(gsub("%", "", Actual)), # there are vars that are thousands and become NA with this line
             Prior  = as.numeric(gsub("%", "", Prior)),
             Revised = as.numeric(gsub("%", "", Revised)))
    
    return(data)
  }
  
  #----------------------------------------------------------------------------------  
  info <- list()
  info$US <- data.frame(Event = c("GDP Annualized QoQ","CPI MoM", "Unemployment Rate",
                                  "Initial Jobless Claims", "Industrial Production MoM", 
                                  "Change in Nonfarm Payrolls","Chicago Fed Nat Activity Index", "Personal Income", "Retail Sales Advance MoM"),
                        Ticker = c("GDPC1", "CPIAUCSL", "UNRATE", "ICSA", "INDPRO", "PAYEMS", "CFNAI", "PI", "SALES"),
                        Freq = c("q","m","m","w","m","m","m", "m", "m"),
                        stringsAsFactors = FALSE)
  
  info$IT <- data.frame(Event = c("GDP WDA QoQ", "CPI EU Harmonized YoY", 
                                  "Industrial Production MoM", "Unemployment Rate Quarterly"),
                        Ticker = c("GDPC1", "CPIAUCSL", "INDPRO", "UNRATE"),
                        Freq = c("q", "m", "m","q"),
                        stringsAsFactors = FALSE)
  
  info$FR <- data.frame(Event = c("Gross Domestic Product (QoQ)", "CPI YoY", 
                                  "Industrial Production MoM", "Unemployment Rate"),
                        Ticker = c("GDPC1", "CPIAUCSL", "INDPRO", "UNRATE"),
                        Freq = c("q", "m", "m","m"),
                        stringsAsFactors = FALSE)
  
  info$UK <- data.frame(Event = c("GDP QoQ", "CPI YoY", 
                                  "Industrial Production MoM", "ILO Unemployment Rate 3Mths"),
                        Ticker = c("GDPC1", "CPIAUCSL", "INDPRO", "UNRATE"),
                        Freq = c("q", "m", "m","m"),
                        stringsAsFactors = FALSE)
  
  info$BE <- data.frame(Event = c("GDP SA QoQ", "CPI YoY","Unemployment Rate"),
                        Ticker = c("GDPC1", "CPIAUCSL",  "UNRATE"),
                        Freq = c("q", "m", "m"),
                        stringsAsFactors = FALSE)
  
  info$NL <- data.frame(Event = c("GDP QoQ", "CPI YoY", 
                                  "Manufacturing Production MoM", "Unemployment Rate"),
                        Ticker = c("GDPC1", "CPIAUCSL", "INDPRO", "UNRATE"),
                        Freq = c("q", "m", "m","m"),
                        stringsAsFactors = FALSE)
  
  info$ES <- data.frame(Event = c("GDP QoQ", "CPI EU Harmonised YoY", 
                                  "Industrial Output NSA YoY", "Unemployment Change"),
                        Ticker = c("GDPC1", "CPIAUCSL", "INDPRO", "UNRATE"),
                        Freq = c("q", "m", "m","m"),
                        stringsAsFactors = FALSE)
  
  info$DE <- data.frame(Event = c("GDP SA QoQ", "CPI EU Harmonized YoY", 
                                  "Industrial Production SA MoM", "Unemployment Change (000's)"),
                        Ticker = c("GDPC1", "CPIAUCSL", "INDPRO", "UNRATE"),
                        Freq = c("q", "m", "m","m"),
                        stringsAsFactors = FALSE)
  
  info$PT <- data.frame(Event = c("GDP QoQ", "CPI YoY", 
                                  "Industrial Production MoM", "Unemployment Rate"),
                        Ticker = c("GDPC1", "CPIAUCSL", "INDPRO", "UNRATE"),
                        Freq = c("q", "m", "m","m"),
                        stringsAsFactors = FALSE)
  
  #----------------------------------------------------------------------------------  
  
  release_dates <- suppressMessages(combine_files(country, folder.raw)) 
  
  
  if (country == "IT")
  {
    # add the same variable with has different "Event"
    
    # cpi
    index <- which(release_dates$Event == "CPI - EU Harmonized (YoY)")
    release_dates[index,"Event"] <- "CPI EU Harmonized YoY"
    
    # indpro
    index <- which(release_dates$Event == "Ind. Prod. sa (MoM)")
    release_dates[index,"Event"] <- "Industrial Production MoM"
    
    # unemployment rate 
    index <- which(release_dates$Event == "Unemployment Rate (s.a)")
    release_dates[index,"Event"] <- "Unemployment Rate Quarterly"
    
  }
  
  
  folder_save <- paste0(folder.save, country,"/")
  
  for (i in 1:nrow(info[[country]]))
  {
    
    event  <- info[[country]][i,"Event"]
    ticker <- info[[country]][i,"Ticker"]
    
    X <- 
      release_dates %>%
      filter(Event == event) %>%
      select(date, country, Event, ref_month, Actual) %>%
      rename(realtime_period = date,
             ref_date        = ref_month,
             !!ticker       := Actual) %>%
      arrange(realtime_period)
    
    if (info[[country]]$Freq[i] == "q")
    {
      X = X %>%
        mutate(quarter = substr(ref_date, 1, 2)) %>%
        mutate(ref_date = if_else(quarter == "1Q", 
                                  ymd(paste0(year(realtime_period),"03-31")),
                                  if_else(quarter == "2Q",
                                          ymd(paste0(year(realtime_period),"06-30")),
                                          if_else(quarter == "3Q",
                                                  ymd(paste0(year(realtime_period),"09-30")),
                                                  ymd(paste0(year(realtime_period)-1,"12-31")))))) %>%
        group_by(ref_date) %>%
        arrange(realtime_period) %>%
        filter(row_number() == 1) %>%
        ungroup %>%
        mutate(diff = realtime_period - ref_date,
               ref_date = if_else(diff < 0, ref_date - years(1), ref_date)) %>%
        select(-quarter, -diff)
    }
    
    
    if (info[[country]]$Freq[i] == "m")
    {
      X = X %>%
        mutate(ref_date = substr(ref_date, 1,3)) %>%
        mutate(ref_date = if_else(ref_date == "Dec",
                                  ymd(paste0(lubridate::year(realtime_period)-1, 
                                             ref_date, 
                                             "01")),
                                  if_else((ref_date == "Nov") & (month(realtime_period) == 1),
                                          ymd(paste0(lubridate::year(realtime_period)-1, 
                                                     ref_date, 
                                                     "01")),
                                          ymd(paste0(lubridate::year(realtime_period), 
                                                     ref_date, 
                                                     "01"))))) %>%
        mutate(ref_date = ymd(paste0(year(ref_date), "-", month(ref_date), "-", days_in_month(ref_date)))) %>%
        group_by(ref_date) %>%
        arrange(realtime_period) %>%
        filter(row_number() == 1) %>%
        ungroup()
      
      # X <-  X %>%
      #         mutate(pub_lag = realtime_period - ref_date) %>% 
      #         mutate(realtime_period = if_else(pub_lag < 10,
      #                                   realtime_period + days_in_month(realtime_period),
      #                                   realtime_period)) %>% select(-pub_lag)
    }
    
    if (info[[country]]$Freq[i] == "w")
    {
      X = X %>%
        mutate(month_ref = substr(ref_date, nchar(ref_date)-2, nchar(ref_date)),
               month_rel = month(realtime_period, abbr = TRUE, label = TRUE)) %>%
        mutate(ref_date = if_else((month_rel == "Jan") &(month_ref == "Dec"),
                                  ydm(paste0(year(realtime_period)-1,"-", ref_date)),
                                  ydm(paste0(year(realtime_period), "-", ref_date))))  %>%
        select(-month_ref, -month_rel)
    }
    
    file.name = paste0(country, "_", ticker,".csv")
    write.csv(X, paste0(folder_save, file.name), row.names = FALSE)
  }
  
  print("Finished.")
}


# LASSO selection of predictors  ---------------------------------------------------------------------------------------------------
#
#  using GLMNET package
#
#  choose lambda by CV, not controlling for nvar
#  RETURNS an equation with the best model

#-----------------------------------------------------------------------------------------------------------------------------------

lasso_arxn_model <- function(dep1, dep, indep, sent,  data, ar.max = 6, x.max = 6, nvar = 8, alpha = 1, 
                             crit = "cv", crossval = FALSE, force.in = NULL, mix.freq = TRUE)
{
  # this function selects variables based on CV
  # possible criteria: crit = c("cv", "bic", "aic", "aicc", "hqc")
  
  n.reg <- 1 + ar.max + 5 * length(indep) + length(sent)
  
  model1       <- define_model(dep, indep, sent, ar.max, x.max, mix.freq)
  
  if (is.null(ar.max)){
    if (is.null(sent))  model.subset <- as.formula(paste0(dep1," ~ ", paste0(model1[3], collapse = "+"))) 
    if (!is.null(sent)) model.subset <- as.formula(paste0(dep1," ~ ", paste0(paste0(model1[3],"+", model1[4]), collapse = "+"))) 
  }
  if (!is.null(ar.max)) model.subset <- as.formula(paste0(dep1," ~ ", paste0(model1[3], collapse = "+"))) 
  
  model.subset <- as.formula(paste0(dep1," ~ ", paste0(model1[1], collapse = "+")))
  fit.mat      <- model.frame(model.subset, data) %>% as.matrix
  
  keep_x <- rep(1, ncol(fit.mat)-1)
  if (length(force.in) > 0) keep_x[force.in] <- 0
  
  nvar <- min(nvar, n.reg)
  N    <- nrow(fit.mat) 
  
  if (crossval == FALSE)
  {
    fit.lasso    <- glmnet(x = fit.mat[,-1], y = fit.mat[,1], dfmax = nvar, penalty.factor = keep_x)
    
    fit.bic     <- log(fit.lasso$dev.ratio^2) + 0.5 * fit.lasso$df * log(N) / N
    fit.df      <- data.frame(bic = fit.bic, df = fit.lasso$df, index = 1:length(fit.bic)) %>% arrange(bic)
    
    if (max(fit.df$df) <= nvar)
    {
      index.lasso <- fit.df[which.min(fit.df$bic),"index"]
    } else {
      index.lasso <- filter(fit.df, df >= nvar)$index[1]
    }
    
    sel.lasso   <- coef(fit.lasso)[,index.lasso]
    sel.lasso   <- names(which(sel.lasso != 0))[-1]
    
    if (length(sel.lasso) > 0)   model.lasso = as.formula(paste0(dep1," ~ ", paste0(sel.lasso, collapse = "+")))
    if (length(sel.lasso) == 0)  model.lasso = as.formula(paste0(dep1," ~ 1"))
  }
  
  if (crossval == TRUE)
  {
    if (crit == "cv")
    {
      cvfit     <- cv.glmnet(x = fit.mat[,-1], y = fit.mat[,1], type.measure = "mse", penalty.factor = keep_x)
      #mylambda  <- cvfit$lambda.min
      index     <- max(which(cvfit$nzero <= nvar))
      mylambda <- cvfit$lambda[index]
      
    }
    if (crit %in% c("bic", "aic", "aicc", "hqc"))
    {
      cvfit     <- ic.glmnet(x= fit.mat[,-1], y = fit.mat[,1], crit = crit,  alpha = alpha, nlambda = 500, dfmax = nvar, penalty.factor = keep_x)
      mylambda  <- cvfit$lambda
      mynvar    <- cvfit$nvar
    }
    
    fit.lasso <- glmnet(x= fit.mat[,-1], y = fit.mat[,1], alpha = alpha, lambda = mylambda, penalty.factor = keep_x)
    sel.lasso <- coef(fit.lasso, s = mylambda)
    sel.lasso <- rownames(sel.lasso)[which(sel.lasso != 0)[-1]]
    
    if (length(sel.lasso) > 0)   model.lasso = as.formula(paste0(dep1," ~ ", paste0(sel.lasso, collapse = "+")))
    if (length(sel.lasso) == 0)  model.lasso = as.formula(paste0(dep1," ~ 1"))
  }
  
  return(model.lasso)
}

##############################################################################################################   
##
##                                  IN-SAMPLE ESTIMATION FUNCTIONS               
##
##############################################################################################################


in_sample_analysis <- function(dep, indep, sent,  mydata = data, lag, release_dates, ar.max = 3, x.max = 3, subset.method = "leaps", 
                               how = "exhaustive", alpha = 1, nvar = 5, ncores = 10, lasso.method = "cv", 
                               ss.method = "adjr2", ar.selection = "yes", force.in = 1, max.var = 20, crossval = TRUE, l0.penalty = "L0", 
                               mix.freq = TRUE)
{
  # FUNCTION TO ESTIMATE THE MODELS IN-SAMPLE
  #
  # subset.method    = c("leaps","ss", "l0","lasso")
  # ss.method = c("adjr2", "bic")
  #
  #
  
  dep1        <- "Y"
  n.horizon   <- length(lag)
  r2          <- data.frame(horizon = lag, AR = NA, ARX = NA, SS = NA, BENCH = NA)
  sigma       <- r2
  store.fit.ar    <- vector("list", n.horizon)
  store.fit.arx   <- vector("list", n.horizon)
  store.fit.ss    <- vector("list", n.horizon)
  store.fit.lasso <- vector("list", n.horizon)
  
  myf <- function(i, dep, dep1, indep, sent, mydata, release_dates, ar.max, 
                  x.max, how, nvar, nfact, alpha, lasso.method, ss.method, ar.selection,
                  force.in, max.var, crossval, subset.method, l0.penalty, mix.freq)
  {
    
    model1  <- define_model(dep, indep, sent, ar.max, x.max, mix.freq)
    r2      <- c()
    sigma   <- c()
    
    Y = mydata %>%
      filter(realtime_period %in% release_dates$realtime_period) %>%
      select(realtime_period, dep) %>%
      rename(Y = dep) %>%
      ungroup
    
    X = mydata %>%
      filter(realtime_period %in% (release_dates$realtime_period - i)) %>%
      mutate(realtime_period = realtime_period + i) %>%
      ungroup
    
    ##--- estimate PRINCIPAL COMPONENTS  of all the macro variables
    # var.q  <-  c("GDPC1", "ULCNFB", "A261RX1Q020SBEA","PCECC96","GPDIC1") #filter(symb.list_all, freq == "q")$symb
    # vars   <- select(X, -realtime_period, -contains("av"), -contains("cum"), -CFNAI, -dep, -var.q) %>% names()
    # vars   <- which(colSums(is.na(X[,vars])) < 10)  %>% vars[.]
    # index  <- which(apply(select(X, vars), 1, function(x) sum(is.na(x))) > 0)
    # temp        <- data.frame(X$realtime_period, matrix(NA, nrow(X), nfact))
    # names(temp) <- c("realtime_period", paste0("pc",1:nfact))
    # if (length(index) > 0)
    # {
    #   pc              <- prcomp(na.omit(X[-index,vars]), center = TRUE, scale = TRUE)
    #   temp[-index,-1] <- pc$x[,1:nfact]
    # } else {
    #    pc        <- prcomp(na.omit(X[,vars]), center = TRUE, scale = TRUE)
    #    temp[,-1] <- pc$x[,1:nfact]
    # }
    # X        <- right_join(X, temp, by = "realtime_period")
    
    reg.data <- inner_join(X, Y, by = "realtime_period")
    
    # make variables used in the regression model to start at the same time
    if (mix.freq)  var.sel <-  c("realtime_period", "Y", dep, indep, c(sapply(indep, function(x) paste0(x, "_", 1:3))), sent)
    if (!mix.freq) var.sel <-  c("realtime_period", "Y", dep, indep, sent)
    estimation <- select(reg.data, var.sel)
    
    index       <- which(apply(estimation, 1, function(x) sum(is.na(x))) > 0)
    if (length(index) > 0) estimation <- estimation[-(index),]
    
    
    # REGRESSIONS   ---------------------------------------------------------------------------------------------------
    
    if (subset.method == "leaps")
    {
      if ((ar.selection == "yes") & (ar.max > 0))
      {
        model.ar  <- leaps_arxn_model(dep1, dep, indep = NULL, sent = NULL, reg.data, ar.max, x.max, how, nvar, ss.method, force.in, crossval) 
        model.arx <- leaps_arxn_model(dep1, dep, indep, sent = NULL, reg.data, ar.max, x.max, how, nvar, ss.method, force.in, crossval)
      } else {
        model.ar  <- as.formula(paste0(dep1, " ~ ", paste0(model1[2], collapse = "+")))
        model.arx <- as.formula(paste0(dep1, " ~ ", paste0(model1[2:3], collapse = "+")))
      }
      model.ss     <- leaps_arxn_model(dep1, dep, indep, sent, reg.data, ar.max, x.max, how, nvar, ss.method, force.in, crossval)
    }
    
    if (subset.method == "ss")
    {
      if (ar.selection == "yes")
      {
        model.ar  <- ss_arxn_model(dep1, dep, indep = NULL, sent = NULL, reg.data, ar.max, x.max, nvar, force.in, max.var)
        model.arx <- ss_arxn_model(dep1, dep, indep, sent = NULL, reg.data, ar.max, x.max, nvar, force.in, max.var)
      } else {
        model.ar  <- as.formula(paste0(dep1, " ~ ", paste0(model1[2], collapse = "+")))
        model.arx <- as.formula(paste0(dep1, " ~ ", paste0(model1[2:3], collapse = "+")))
      }
      model.ss     <- ss_arxn_model(dep1, dep, indep, sent, reg.data, ar.max, x.max, nvar, force.in, max.var)
    }
    if (subset.method == "l0")
    {
      if (ar.selection == "yes")
      {
        model.ar  <- l0_arxn_model(dep1, dep, indep = NULL, sent = NULL, reg.data, ar.max, x.max, nvar, crossval, l0.penalty, force.in)
        model.arx <- l0_arxn_model(dep1, dep, indep,        sent = NULL, reg.data, ar.max, x.max, nvar, crossval, l0.penalty, force.in)
      } else {
        model.ar  <- as.formula(paste0(dep1, " ~ ", paste0(model1[2], collapse = "+")))
        model.arx <- as.formula(paste0(dep1, " ~ ", paste0(model1[2:3], collapse = "+")))
      }
      model.ss     <- l0_arxn_model(dep1, dep, indep, sent, reg.data, ar.max, x.max, nvar, crossval, l0.penalty, force.in)
    }
    if (subset.method == "lasso")
    {
      if (ar.selection == "yes")
      {
        model.ar  <- lasso_arxn_model(dep1, dep, indep = NULL, sent = NULL, reg.data, ar.max, x.max, nvar, alpha, crit = lasso.method, crossval, force.in, mix.freq)
        model.arx <- lasso_arxn_model(dep1, dep, indep, sent = NULL, reg.data, ar.max, x.max, nvar, alpha, crit = lasso.method, crossval, force.in, mix.freq)
      } else  {
        model.ar  <- as.formula(paste0(dep1, " ~ ", paste0(model1[2], collapse = "+")))
        model.arx <- as.formula(paste0(dep1, " ~ ", paste0(model1[2:3], collapse = "+")))
      }
      model.ss     <- lasso_arxn_model(dep1, dep, indep, sent, reg.data, ar.max, x.max, nvar, alpha, crit = lasso.method, crossval, force.in, mix.freq)
    }
    #  ---------------------------------------------------------------------------------------------------
    
    
    fit.ar    <- lm(model.ar,   data = reg.data)
    fit.ar    <- summary(fit.ar)
    fit.arx   <- lm(model.arx,  data = reg.data)
    fit.arx   <- summary(fit.arx)
    fit.ss    <- lm(model.ss,   data = reg.data)
    fit.ss.nw <- coeftest(fit.ss, vcov = vcovHAC(fit.ss))
    fit.ss    <- summary(fit.ss)
    fit.ss$coefficients[,"t value"] <- fit.ss.nw[,"t value"]
    
    r2["AR"]    <- fit.ar$r.squared
    r2["ARX"]   <- fit.arx$r.squared
    r2["SS"]    <- fit.ss$r.squared
    sigma["AR"]    <- fit.ar$sigma
    sigma["ARX"]   <- fit.arx$sigma
    sigma["SS"]    <- fit.ss$sigma
    
    
    # add one sentiment measure at the time to AR/ARX and make ARN/ARXN
    for (j in 1:length(sent))
    {
      model.arn   <- as.formula(paste0(as.character(model.ar), " + ", sent[j]))
      model.arxn  <- as.formula(paste0(as.character(model.arx), " + ", sent[j]))
      
      fit.arn   <- lm(model.arn,  data = reg.data)
      fit.arxn  <- lm(model.arxn, data = reg.data)
      
      r2[paste0("ARN_",sent[j])]      <- summary(fit.arn)$r.squared
      r2[paste0("ARXN_",sent[j])]     <- summary(fit.arxn)$r.squared
      sigma[paste0("ARN_",sent[j])]   <- summary(fit.arn)$sigma
      sigma[paste0("ARXN_",sent[j])]  <- summary(fit.arxn)$sigma
    }
    
    return(list(r2 = r2, sigma = sigma, fit.ar = fit.ar, fit.arx = fit.arx, fit.ss = fit.ss))
    
  } # end myf() -----------------------------------------------------------------------------------
  
  t0 = Sys.time()
  
  use_core <- min(n.horizon, ncores)
  cluster = makeCluster(use_core, type = "SOCK")
  registerDoSNOW(cluster)
  
  pb       <- txtProgressBar(max = n.horizon, style = 3)
  progress <- function(n) setTxtProgressBar(pb, n)
  opts     <- list(progress = progress)
  
  val = foreach(i =1:n.horizon, 
                .combine = 'rbind',
                .packages = c("dplyr", "leaps", "glmnet", "formula.tools", "HDeconometrics", "L0Learn", "lmtest","sandwich"),
                .verbose  = FALSE,
                .export   = c("define_model", "leaps_arxn_model", "lasso_arxn_model", "ss_arxn_model", "l0_arxn_model"),
                .options.snow = opts) %dopar% 
    myf(lag[i], dep, dep1, indep, sent, mydata, release_dates, ar.max, 
        x.max, how, nvar, nfact, alpha, lasso.method, ss.method, ar.selection,
        force.in, max.var, crossval, subset.method, l0.penalty, mix.freq)
  
  close(pb)
  stopCluster(cluster)
  
  t1 = Sys.time() - t0 
  
  r2    <- data.frame(do.call(rbind, val[,1]), row.names = NULL) %>% mutate(horizon = lag)
  sigma <- data.frame(do.call(rbind, val[,2]), row.names = NULL) %>% mutate(horizon = lag)
  
  return(list(r2 = r2, sigma = sigma, fit.ar = val[,3], fit.arx = val[,4], fit.ss = val[,5]))
}

##############################################################################################
##    OUT-OF-SAMPLE FUNCTIONS
##############################################################################################



out_sample_analysis <- function(dep, indep, sent, mydata = data, start.forecast, release_dates, lag, ar.max = 6, x.max = 1, 
                                subset.method = "leaps",how = "exhaustive", scheme = "recursive",
                                alpha = 0.2, nvar = 5, ncores = 20, lasso.method = "cv", 
                                ss.method = "adjr2", ar.selection = "yes", force.in = 1, max.var = 20, crossval = TRUE,
                                benchmark1 = NULL, benchmark2 = NULL, mix.freq = FALSE, symb.list_all)
{
  #
  # function to produce out-of-sample forecasts for the conditional mean
  #
  
  n.horizon      <- length(lag)
  start.index    <- max(which(release_dates$realtime_period <= as.Date(start.forecast)))
  end.index      <- max(which(release_dates$realtime_period <= as.Date("2020-12-31")))
  macro1         <- trimws(unlist(strsplit(unlist(strsplit(benchmark1,"~"))[2], split ="+", fixed = TRUE)))
  macro2         <- trimws(unlist(strsplit(unlist(strsplit(benchmark2,"~"))[2], split ="+", fixed = TRUE)))
  #--------------------------------------------------------------------------------------------------------
  # function that produces the forecasts for all models at a given horizon
  
  myf <- function(horizon, mydata, dep, indep, sent, release_dates, start.index, end.index,  
                  scheme, ar.max, x.max, how, nvar, lasso.method, ss.method, ar.selection,
                  force.in, max.var, crossval, subset.method, symb.list_all)
  {
    
    dep1         <- "Y"
    model1       <- define_model(dep, indep, sent, ar.max, x.max, mix.freq, symb.list_all)
    store.forecast <- data.frame()
    store.ss       <- vector("list", end.index-start.index)
    store.arx      <- store.ss
    store.news     <- store.ss
    store.bench1   <- store.ss
    store.bench2   <- store.ss
    
    for (t in start.index:(end.index-1))
    {
      
      if (scheme == "recursive") target.date <- 1:(t+1)
      if (scheme == "rolling")   target.date <- (t-start.index+1):(t+1)
      
      Y.target =
        mydata %>% 
        filter(realtime_period %in% (release_dates$realtime_period[target.date])) %>% 
        select(realtime_period, dep) %>%
        rename(Y = dep) %>%
        ungroup
      
      X.target =
        mydata %>% 
        filter(realtime_period %in% (release_dates$realtime_period[target.date]- horizon)) %>% 
        mutate(realtime_period = realtime_period + horizon) %>% 
        ungroup
      
      # code to calculate PCs ..................................................................................
      
      # var.q  <-  c("GDPC1", "ULCNFB", "A261RX1Q020SBEA","PCECC96","GPDIC1") #filter(symb.list, freq == "q")$symb
      # vars   <- select(X.target, -realtime_period, -contains("av"), -contains("cum"), -CFNAI, -dep, -var.q) %>% names()
      # vars   <- which(colSums(is.na(X.target[,vars])) < 10)  %>% vars[.]
      # index  <- which(apply(select(X.target, vars), 1, function(x) sum(is.na(x))) > 0)
      # 
      # temp        <- data.frame(X.target$realtime_period, matrix(NA, nrow(X.target), nfact))
      # names(temp) <- c("realtime_period", paste0("pc",1:nfact))
      # 
      # if (length(index) > 0)
      # {
      #   pc              <- prcomp(na.omit(X.target[-index,vars]), center = TRUE, scale = TRUE)
      #   temp[-index,-1] <- pc$x[,1:nfact]
      # } else {
      #   pc        <- prcomp(na.omit(X.target[,vars]), center = TRUE, scale = TRUE)
      #   temp[,-1] <- pc$x[,1:nfact]
      # }
      # 
      # X.target        <- right_join(X.target, temp, by = "realtime_period")
      
      # ............................................................................................................
      
      X.estimation <- X.target[-nrow(X.target),]
      Y.estimation <- Y.target[-nrow(Y.target),]
      
      estimation  <- inner_join(X.estimation, Y.estimation, by = 'realtime_period')
      target      <- inner_join(X.target, Y.target, by = "realtime_period")
      
      # make variables used in the regression model to start at the same time
      #      var.sel     <-  c("realtime_period", "Y", dep, indep, c(sapply(indep, function(x) paste0(x, "_", 1:3))), sent, 
      #                        "CFNAI", macro1, macro2)
      
      # make variables used in the regression model to start at the same time
      if (mix.freq)  var.sel <-  c("realtime_period", "Y", dep, indep, c(sapply(indep, function(x) paste0(x, "_", 1:3))), sent, macro1, macro2)
      if (!mix.freq) var.sel <-  c("realtime_period", "Y", dep, indep, sent, macro1, macro2)
      estimation <- select(estimation, var.sel)
      
      index <- which(apply(estimation, 1, function(x) sum(is.na(x))) > 0)
      if (length(index) > 0) estimation <- estimation[-(index),]
      
      # REGRESSIONS   ---------------------------------------------------------------------------------------------------
      
      if (subset.method == "leaps")
      {
        if ((ar.selection == "yes") & (ar.max > 0))
        {
          model.ar  <- leaps_arxn_model(dep1, dep, indep = NULL, sent = NULL, estimation, ar.max, x.max, how, nvar, ss.method, force.in, crossval) 
          model.arx <- leaps_arxn_model(dep1, dep, indep, sent = NULL, estimation, ar.max, x.max, how, nvar, ss.method, force.in, crossval)
        }
        else 
        {
          model.ar  <- as.formula(paste0(dep1, " ~ ", paste0(model1[2], collapse = "+"))) 
          model.arx <- leaps_arxn_model(dep1, dep, indep, sent = NULL, estimation, ar.max, x.max, how, nvar, ss.method, force.in, crossval)
        }
        
        model.ss     <- leaps_arxn_model(dep1, dep, indep,        sent, estimation, ar.max, x.max, how, nvar, ss.method, force.in, crossval)
        model.news   <- leaps_arxn_model(dep1, dep, indep = NULL, sent, estimation, ar.max, x.max, how, nvar, ss.method, force.in, crossval)
      }
      
      if (subset.method == "ss")
      {
        if (ar.selection == "yes")
        {
          model.ar  <- ss_arxn_model(dep1, dep, indep = NULL, sent = NULL, estimation, ar.max, x.max, nvar, force.in, max.var)
          model.arx <- ss_arxn_model(dep1, dep, indep, sent = NULL, estimation, ar.max, x.max, nvar, force.in, max.var)
        }
        else 
        {
          model.ar  <- as.formula(paste0(dep1, " ~ ", paste0(model1[2], collapse = "+")))
          model.arx <- ss_arxn_model(dep1, dep, indep, sent = NULL, estimation, ar.max, x.max, nvar, force.in, max.var)
        }
        
        model.ss     <- ss_arxn_model(dep1, dep, indep,        sent, estimation, ar.max, x.max, nvar, force.in, max.var)
        model.news   <- ss_arxn_model(dep1, dep, indep = NULL, sent, estimation, ar.max, x.max, nvar, force.in, max.var)
      }
      if (subset.method == "l0")
      {
        if (ar.selection == "yes")
        {
          model.ar  <- l0_arxn_model(dep1, dep, indep = NULL, sent = NULL, estimation, ar.max, x.max, nvar, crossval = TRUE, l0.penalty, force.in)
          model.arx <- l0_arxn_model(dep1, dep, indep,        sent = NULL, estimation, ar.max, x.max, nvar, crossval = TRUE, l0.penalty, force.in)
        }
        else 
        {
          model.ar  <- as.formula(paste0(dep1, " ~ ", paste0(model1[2], collapse = "+")))
          model.arx <- l0_arxn_model(dep1, dep, indep,        sent = NULL, estimation, ar.max, x.max, nvar, crossval = TRUE, l0.penalty, force.in)
        }
        
        model.ss     <- l0_arxn_model(dep1, dep, indep,        sent, estimation, ar.max, x.max, nvar, crossval = TRUE, l0.penalty, force.in)
        model.news   <- l0_arxn_model(dep1, dep, indep = NULL, sent, estimation, ar.max, x.max, nvar, crossval = TRUE, l0.penalty, force.in)
      }
      if (subset.method == "lasso")
      {
        if (ar.selection == "yes")
        {
          model.ar  <- lasso_arxn_model(dep1, dep, indep = NULL, sent = NULL, estimation, ar.max, x.max, nvar, alpha, crit = lasso.method, crossval, force.in, mix.freq)
          model.arx <- lasso_arxn_model(dep1, dep, indep, sent = NULL, estimation, ar.max, x.max, nvar, alpha, crit = lasso.method, crossval, force.in,mix.freq)
        }
        else 
        {
          model.ar  <- as.formula(paste0(dep1, " ~ ", paste0(model1[2], collapse = "+")))
          model.arx <- lasso_arxn_model(dep1, dep, indep, sent = NULL, estimation, ar.max, x.max, nvar, alpha, crit = lasso.method, crossval, force.in, mix.freq)
        }
        
        model.ss     <- lasso_arxn_model(dep1, dep, indep, sent, estimation, ar.max, x.max, nvar, alpha, crit = lasso.method, crossval, force.in, mix.freq)
        #model.news   <- lasso_arxn_model(dep1, dep, indep = NULL, sent, estimation, ar.max, x.max, nvar, alpha, crit=lasso.method, crossval, force.in, mix.freq)
        model.news <- model.ar
      }
      
      #  ---------------------------------------------------------------------------------------------------
      
      if (is.character(benchmark1)) 
      {
        fit.bench1      <- lm(benchmark1, data = estimation)  
        forecast.bench1 <- predict(fit.bench1, newdata = target) %>% tail(1) %>% c()
        names(forecast.bench1)  <- "BENCH1"
      }
      if (is.character(benchmark2)) 
      {
        fit.bench2      <- lm(benchmark2, data = estimation)  
        forecast.bench2 <- predict(fit.bench2, newdata = target) %>% tail(1) %>% c()
        names(forecast.bench2)  <- "BENCH2"
      }
      
      fit.ar     <- lm(model.ar,  data = estimation)
      fit.arx    <- lm(model.arx, data = estimation)
      fit.ss     <- lm(model.ss,  data = estimation)
      fit.news   <- lm(model.news, data = estimation)
      
      store.ss[[t-start.index+1]]   <- summary(fit.ss)
      store.arx[[t-start.index+1]]  <- summary(fit.arx)
      store.news[[t-start.index+1]] <- summary(fit.news)
      store.bench1[[t-start.index+1]] <- summary(fit.bench1)
      store.bench2[[t-start.index+1]] <- summary(fit.bench2)
      
      forecast.ar   <- predict(fit.ar, newdata = target)  %>% tail(1) %>% c()
      names(forecast.ar)  <- "AR"
      forecast.arx  <- predict(fit.arx, newdata = target) %>% tail(1) %>% c()
      names(forecast.arx) <- "ARX"
      forecast.ss   <- predict(fit.ss, newdata = target)  %>% tail(1) %>% c()
      names(forecast.ss)  <- "SS"
      forecast.news   <- predict(fit.news, newdata = target)  %>% tail(1) %>% c()
      names(forecast.news)  <- "NEWS"
      
      temp_arn   <- c()
      temp_arxn  <- c()
      temp_arpcn <- c()
      
      for (j in 1:length(sent))
      {
        model.arn   <- as.formula(paste0(as.character(model.ar), "+ " , sent[j]))
        model.arxn  <- as.formula(paste0(as.character(model.arx), "+ " , sent[j]))
        
        fit.arn   <- lm(model.arn,  data = estimation)
        fit.arxn  <- lm(model.arxn, data = estimation)
        
        forecast.arn   <- predict(fit.arn, newdata = target)  %>% tail(1)
        forecast.arxn  <- predict(fit.arxn, newdata = target) %>% tail(1)
        
        names(forecast.arn)   <- paste0("ARN_",sent[j])
        names(forecast.arxn)  <- paste0("ARXN_",sent[j])
        
        temp_arn   <- c(temp_arn, forecast.arn)
        temp_arxn  <- c(temp_arxn, forecast.arxn)
      }
      
      realization      <- tail(target$Y,1)
      realization.date <- as.character(tail(target$realtime_period,1))
      forecast  <- c(forecast.bench1, forecast.bench2, forecast.ar, forecast.arx, forecast.ss, forecast.news, temp_arn, temp_arxn)
      results   <- data.frame(Y = realization,
                              realtime_period = realization.date, 
                              t(forecast),
                              horizon = horizon,
                              stringsAsFactors = FALSE)
      
      store.forecast   <- rbind(store.forecast, results)
    }
    
    return(list(store.forecast, store.ss, store.arx, store.news, store.bench1, store.bench2))
  }
  # end
  #-----------------------------------------------------------------------------------------------------------
  
  # #  serial implementation
  #  store.forecast <- vector("list", length(horizon.plot))
  #  for (i in 1:length(horizon.plot)) store.forecast[[i]] = myf(horizon.plot[i], mydata, dep, indep, sent, release_dates,
  #                                                              start.index, end.index, ar.max, x.max, how, nvar, lasso.method, ss.method)
  
  t0 = Sys.time()
  
  use_cores <- min(n.horizon, ncores)
  cluster = makeCluster(use_cores, type = "SOCK")
  registerDoSNOW(cluster)
  
  pb       <- txtProgressBar(max = n.horizon, style = 3)
  progress <- function(n) setTxtProgressBar(pb, n)
  opts     <- list(progress = progress)
  
  val = foreach(i = 1:n.horizon, 
                .combine = 'rbind',
                .packages = c("dplyr", "leaps", "glmnet", "formula.tools", "HDeconometrics", "L0Learn"),
                .verbose  = FALSE,
                .export   = c("define_model", "lasso_arxn_model", "leaps_arxn_model", "ss_arxn_model", "l0_arxn_model"),
                .options.snow = opts) %dopar% 
    myf(lag[i], mydata, dep, indep, sent, release_dates, 
        start.index, end.index, scheme, ar.max, x.max, how, nvar,
        lasso.method, ss.method, ar.selection,
        force.in, max.var, crossval, subset.method, symb.list_all)
  
  close(pb)
  stopCluster(cluster)
  
  t1 = Sys.time() - t0  
  
  store.forecast   <- data.frame(do.call(rbind, val[,1]))
  store.forecast$realtime_period = as.Date(store.forecast$realtime_period)
  
  
  return(list(forecast = store.forecast, fit.ss = val[,2], fit.arx = val[,3], 
              fit.news = val[,4], fit.bench1= val[,5], fit.bench2 = val[,6]))  
}





plot_outsample <- function(store.forecast, store.error, mpe, mpe.ratio, var, hor.plot, metric = "RMSPE")
{
  
  ## ---------------------------------------------------------------------------------------   
  ## plotting
  
  title <- data.frame(dep   = c("UNRATE", "PAYEMS", "INDPRO","GDPC1"), 
                      title = c("Unemployment Rate", "Nonfarm Payroll Employment","Industrial Production", "Real GDP"),
                      stringsAsFactors = FALSE)
  title <- title[title$dep == dep, "title"]
  
  rmspe.abs.plot <- 
    mpe %>%
    #filter(Model != "DFM") %>%
    mutate(bench = substr(Model, 1, 3),
           bench = plyr::mapvalues(bench, 
                                   from = c("AR", "SS_", "ARX","BENCH"),
                                   to   = c("ARN" , "SS", "ARXN", "BENCH"), 
                                   warn_missing = FALSE)) %>%
    gather(Measure, Value, -Model, -horizon, -bench) %>%
    ggplot(.) +
    geom_rect(data=data.frame(xmin = 28, xmax = 112, ymin=-Inf, ymax=Inf), 
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey95",  inherit.aes = FALSE)  +
    geom_line(aes(horizon, Value, group = Model), color = "gray50") + 
    labs(x = NULL, y = NULL, title = title, subtitle = "Error measures by horizon") +
    theme_bw() + 
    theme(legend.position = "none") + 
    #geom_line(data = filter(mpe,  Model == "DFM"),  aes(horizon, MSPE),  color = 'black', size = 0.5, linetype = "dashed") +
    #geom_line(data = filter(mpe,  Model == "BENCH"),  aes(horizon, MSPE),  color = 'black', size = 0.5, linetype = "dashed") +
    #geom_line(data = filter(select(mpe, Model, horizon, RMSPE), Model == "ARX"), aes(horizon, RMSPE),  color = 'steelblue4', size = 0.8, linetype = "twodash") +
    scale_x_reverse() + guides(colour = guide_legend(nrow = 1)) + 
    facet_grid(Measure ~ bench, scales = "free") #+ ylim(1,2.5)
  
  
  rmspe.abs.plot2 <- 
    mpe %>%
    select(Model, horizon, RMSPE) %>%
    spread(Model, RMSPE) %>%
    gather(Model, Value, -horizon, -AR, -ARX, -BENCH, -DFM) %>%
    mutate(bench = substr(Model, 1, 3),
           bench = plyr::mapvalues(bench, 
                                   from = c("AR", "SS_", "ARX"),
                                   to   = c("ARN" , "SS", "ARXN"), 
                                   warn_missing = FALSE)) %>%
    gather(Measure, Value, -Model, -horizon, -bench, -AR, -ARX, -BENCH, -DFM) %>%
    ggplot(.) +
    geom_rect(data=data.frame(xmin = 28, xmax = ifelse(max(mpe$horizon) < 112, max(mpe$horizon), 112), ymin=-Inf, ymax=Inf), 
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey95",  inherit.aes = FALSE)  +
    geom_point(aes(horizon, Value, group = Model), color = "gray60", size = 1) + 
    geom_line(aes(horizon, Value, group = Model), color = "gray60", linetype = "dotted") + 
    #geom_line(aes(horizon, AR, group = Model, color = "AR"), linetype = "dashed") + 
    geom_line(aes(horizon, ARX, group = Model, color = "ARX"), linetype = "solid") + 
    geom_line(aes(horizon, BENCH, group = Model,color = "BENCH"), linetype = "solid") + 
    geom_line(aes(horizon, DFM, group = Model, color = "DFM"), linetype = "solid") + 
    scale_colour_manual("", 
                        breaks = c("AR", "ARX", "BENCH","DFM"),
                        values = c("black", "tomato4", "dodgerblue2", "yellow4")) +
    labs(x = NULL, y = NULL, title = title, subtitle = "RMSPE by horizon (ARN/ARXN vs others)") +
    theme_bw() + 
    theme(legend.position = "bottom") + 
    #geom_line(data = filter(mpe,  Model == "DFM"),  aes(horizon, MSPE),  color = 'black', size = 0.5, linetype = "dashed") +
    #geom_line(data = filter(mpe,  Model == "BENCH"),  aes(horizon, MSPE),  color = 'black', size = 0.5, linetype = "dashed") +
    #geom_line(data = filter(select(mpe, Model, horizon, RMSPE), Model == "ARX"), aes(horizon, RMSPE),  color = 'steelblue4', size = 0.8, linetype = "twodash") +
    scale_x_reverse() + guides(colour = guide_legend(nrow = 1)) + 
    facet_grid( ~ bench, scales = "free") #+ ylim(1,2.5)
  
  
  rmspe.rel.plot <- 
    mpe.ratio %>%
    mutate(bench = substr(Model, 1, 3),
           bench = plyr::mapvalues(bench, 
                                   from = c("AR", "SS_", "ARX", "BEN", "ave","med"), 
                                   to = c("ARN" , "SS", "ARXN", "BENCHMARK","average", "median"),
                                   warn_missing = FALSE)) %>%
    gather(Measure, Value, -Model, -horizon, -bench) %>%
    ggplot(.) + 
    geom_rect(data=data.frame(xmin = 28, xmax = 112, ymin=-Inf, ymax=Inf), 
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey95",  inherit.aes = FALSE)  +
    geom_line(aes(horizon, Value, group = Model, color=Model)) + 
    labs(x = NULL, y = NULL, title = title, subtitle = "RMSPE relative to ARX") +
    theme_bw() + 
    theme(legend.position = "none", panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + 
    #geom_line(data = filter(select(mpe.ratio, horizon, Model, RMSPE), Model == "AR"), aes(horizon, RMSPE, group=Model),  color = 'black', size = 1, linetype = "dashed") + 
    geom_hline(yintercept = 1, color = "steelblue4", size = 1, linetype = "twodash") + 
    scale_x_reverse() + guides(colour = guide_legend(nrow = 1)) + 
    facet_grid(Measure ~ bench, scales = "free") 
  
  
  # rolling errors
  
  if (var == "GDPC1") roll.window = 8 else roll.window = 36  
  roll.mean.forecast <- rollify(mean, roll.window) # the data are now monthly
  
  rmspe.roll <- store.error %>%
    select(-Y) %>%
    group_by(horizon) %>%
    mutate_at(vars(-c(realtime_period, horizon)), funs(roll.mean.forecast(abs(.)^2))) %>%
    mutate_at(vars(-c(realtime_period, horizon, ARX)), funs((. / ARX)^0.5)) %>% 
    select(-ARX) %>%
    ungroup %>%
    tidyr::gather(Model, RMSPE_roll, -realtime_period, -horizon)
  
  mape.roll <- store.error %>%
    select(-Y) %>%
    group_by(horizon) %>%
    mutate_at(vars(-c(realtime_period, horizon)), funs(roll.mean.forecast(abs(.)))) %>%
    mutate_at(vars(-c(realtime_period, horizon, ARX)), funs((. / ARX)^0.5)) %>% 
    select(-ARX) %>%
    ungroup %>%
    tidyr::gather(Model, MAPE_roll, -realtime_period, -horizon)
  
  mspe.roll <- store.error %>%
    select(-Y) %>%
    group_by(horizon) %>%
    mutate_at(vars(-c(realtime_period, horizon)), funs(roll.mean.forecast(abs(.)^2))) %>%
    mutate_at(vars(-c(realtime_period, horizon, ARX)), funs((. / ARX)^0.5)) %>% 
    select(-ARX) %>%
    ungroup %>%
    tidyr::gather(Model, MSPE_roll, -realtime_period, -horizon)
  
  error.roll <- rmspe.roll %>% 
    left_join(mape.roll, by = c("realtime_period", "horizon","Model")) %>%
    left_join(mspe.roll, by = c("realtime_period", "horizon","Model"))
  
  
  best.models <- sort(colMeans(abs(store.error[,5:ncol(store.error)]))) %>% names %>% head(3)
  
  all.roll.plot <- 
    error.roll %>%
    mutate(bench = substr(Model, 1, 3),
           bench = plyr::mapvalues(bench, 
                                   from = c("AR", "ARP", "LAS", "SS_", "ARX"),
                                   to   = c("ARN" ,"ARPC", "LASSO", "SS", "ARXN"), 
                                   warn_missing = FALSE)) %>%
    #filter(bench %in% c("ARN", "ARXN", "SS")) %>%
    gather(Measure, Value, -realtime_period, -horizon, -Model,-bench) %>%
    filter(Model != "ARX") %>%
    filter(horizon %in% hor.plot) %>%
    ggplot() +
    geom_rect(data=(data.frame(xmin=nber.min, xmax=nber.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="pink",alpha=0.3) +
    geom_line(aes(realtime_period, Value, group = Model, color = Model)) +
    geom_hline(yintercept = 1, color = "steelblue4", linetype = "twodash") + 
    theme_bw() + 
    labs(x = NULL, Y = "MAPE", title = paste0(title, " (RMSPE relative to ARPC)")) +
    theme(legend.position = "none") + 
    facet_grid(Measure ~ horizon, scales = "free")
  
  
  rmspe.roll.plot <- 
    rmspe.roll %>%
    spread(Model, RMSPE_roll) %>%
    gather(Model, Value, -horizon, -realtime_period, -AR, -BENCH, -DFM, -average_all, -average_arn, -average, -median_all, -median_arn, -median_arxn) %>%
    mutate(bench = substr(Model, 1, 3),
           bench = plyr::mapvalues(bench, 
                                   from = c("AR", "ARP", "LAS", "SS_", "ARX"),
                                   to   = c("ARN" ,"ARPC", "LASSO", "SS", "ARXN"), 
                                   warn_missing = FALSE)) %>%
    gather(Measure, Value, -Model, -realtime_period, -horizon, -bench, -AR,  -BENCH, -DFM,  -average_all, 
           -average_arn, -average, -median_all, -median_arn, -median_arxn) %>%
    filter(horizon %in% hor.plot) %>%
    ggplot() +
    geom_rect(data=(data.frame(xmin=nber.min, xmax=nber.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="pink",alpha=0.3) +
    geom_line(aes(realtime_period, Value, group = Model), color = "gray70") +
    geom_line(aes(realtime_period, average_all, group = Model, color = "ave")) +
    geom_line(aes(realtime_period, BENCH, group = Model, color = "bench")) +
    geom_line(aes(realtime_period, median_all, group = Model, color = "med")) +
    geom_line(aes(realtime_period,DFM, group = Model, color = "DFM")) +
    geom_hline(yintercept = 1, color = "steelblue4", linetype = "twodash") + 
    theme_bw() + 
    scale_colour_manual("", 
                        breaks = c("ave", "bench", "med","DFM"),
                        values = c("yellow4", "black", "tomato4", "dodgerblue2")) +
    labs(x = NULL, Y = NULL, title = paste0(title, " (RMSPE relative to ARX)")) +
    theme(legend.position = "bottom") + 
    facet_grid(bench ~ horizon, scales = "free")
  
  
  return(list(rmspe.abs.plot2, rmspe.rel.plot,  rmspe.roll.plot, all.roll.plot))   
}

#############################           DIEBOLD-MARIANO TEST STATISTIC

#    bench = error of the benchmark model 
#    alt   = error of the alternative model
#     h    = forecast horizon
#   power  = loss function 

myDM <- function(bench, alt, h = 2, power = 2)
{
  alt   <- as.numeric(alt)
  bench <- as.numeric(bench)
  n     <- length(bench)
  
  # alt   = ifelse(abs(alt) > mean(alt, na.rm=T) + 4 * sd(alt, na.rm = T), NA, alt)
  #  bench = ifelse(abs(bench) > mean(bench, na.rm=T) + 4 * sd(bench, na.rm = T), NA, bench)
  
  diff = abs(alt)^power - abs(bench)^power
  
  if (!is.na(var(diff, na.rm = TRUE)))
  {
    if(var(diff, na.rm = T) > 0)
    {
      fit <- lm(diff ~ 1)
      mycov <- sandwich::NeweyWest(fit, lag = h, prewhite = FALSE)
      fit <- lmtest::coeftest(fit, vcov = mycov)
      DM  <- fit[1,3] 
    } else { DM <- 0}
  } else { DM <- NA}
  
  ## correction Harvey, Leybourne, Newbold
  
  correction = ((n + 1 - 2*h + h*(h-1)/n) / n)^0.5
  DMc <- correction * DM
  
  return(ciao = data.frame(DM = DM, pDM = pnorm(DM), DMc = DMc, pDMc = pt(DMc, n-1)))
}

#############################           GRANZIERA-SEKHPOSYAN TEST STATISTIC

#    bench = error of the benchmark model 
#    alt   = error of the alternative model
#     h    = forecast horizon
#   power  = loss function 
#   conditional on NBER recessions

myDMcond <- function(bench, alt, X,  h=2)
{
  alt   <- as.numeric(alt)
  bench <- as.numeric(bench)
  X     <- as.numeric(X)
  n     <- length(bench)
  
  diff = abs(alt)^2 - abs(bench)^2
  
  if (!is.na(var(diff, na.rm = TRUE)))
  {
    if(var(diff, na.rm = T) > 0)
    {
      
      fit <- lm(diff ~ X)
      mycov <- sandwich::NeweyWest(fit, lag = h, prewhite = FALSE)
      test <- car::linearHypothesis(fit, c("(Intercept)","X"), vcov. = mycov, test="Chisq")
      DMp  <- test$`Pr(>Chisq)`[2]
      DM   <- test$Chisq[2]
      
      # test <- car::linearHypothesis(fit, c("(Intercept)","X"), vcov. = mycov, test="F")
      # DMp  <- test$`Pr(>F)`[2]
      # DM   <- test$F[2]
      IGW  <- mean(fitted(fit) >= 0)  # IGW formula from the paper 
      MGW  <- sum(abs(fitted(fit)) * (fitted(fit) >= 0)) / sum(abs(fitted(fit)))
    } else { DM <- 0}
  } else { DM <- NA}
  
  results = list(df = data.frame(DM = DM, pDM = DMp, IGW = IGW, MGW = MGW), fit = fit)
  
  return(results)
}


qDM <- function(bench, alt, h = 2)
{
  alt   <- as.numeric(alt)
  bench <- as.numeric(bench)
  
  alt   = ifelse(abs(alt) > mean(alt, na.rm=T) + 4 * sd(alt, na.rm = T), NA, alt)
  bench = ifelse(abs(bench) > mean(bench, na.rm=T) + 4 * sd(bench, na.rm = T), NA, bench)
  
  diff = alt - bench # if negative, e2 beats benchmark
  if(var(diff, na.rm = T) > 0)
  {
    fit <- lm(diff ~ 1)
    mycov <- sandwich::NeweyWest(fit, lag = h)
    fit <- lmtest::coeftest(fit, vcov = mycov)
    DM  <- fit[1,3] 
  } else { DM <- 0}
  
  return(DM)
}

qDM1 <- function(bench, alt, h = 2, power = 2)
{
  alt   <- as.numeric(alt)
  bench <- as.numeric(bench)
  n     <- length(bench)
  
  # alt   = ifelse(abs(alt) > mean(alt, na.rm=T) + 4 * sd(alt, na.rm = T), NA, alt)
  #  bench = ifelse(abs(bench) > mean(bench, na.rm=T) + 4 * sd(bench, na.rm = T), NA, bench)
  
  diff = abs(alt)^power - abs(bench)^power
  
  if (!is.na(var(diff, na.rm = TRUE)))
  {
    if(var(diff, na.rm = T) > 0)
    {
      fit <- lm(diff ~ 1)
      mycov <- sandwich::NeweyWest(fit, lag = h, prewhite = FALSE)
      fit <- lmtest::coeftest(fit, vcov = mycov)
      DM  <- fit[1,3] 
    } else { DM <- 0}
  } else { DM <- NA}
  
  ## correction Harvey, Leybourne, Newbold
  
  correction = ((n + 1 - 2*h + h*(h-1)/n) / n)^0.5
  DMc <- correction * DM
  
  return(DM)
}

myfluctuation_test <- function (loss1, loss2, mu = 0.5, lag_truncate = 4, time_labels = NULL, conf_level = 0.05) 
{
  library(murphydiagram)
  
  if (length(loss1) != length(loss2)) {
    stop("Vectors of losses must have the same length")
  }
  if (all(abs(seq(from = 0.1, to = 0.9, by = 0.1) - mu) > 1e-12)) {
    stop("mu must be in {0.1, 0.2, ..., 0.9}")
  }
  if (!is.null(time_labels) & length(time_labels) != length(loss1)) {
    warning("Specified time labels are inconsistent - simple integers used instead")
    time_labels <- NULL
  }
  if (!conf_level %in% c(0.05, 0.1)) {
    stop("significance_level must be either 0.05 or 0.1")
  }
  CV <- cbind(seq(from = 0.1, to = 0.9, by = 0.1), c(3.393, 
                                                     3.179, 3.012, 2.89, 2.779, 2.634, 2.56, 2.433, 2.248), 
              c(3.17, 2.948, 2.766, 2.626, 2.5, 2.356, 2.252, 2.13, 
                1.95))
  
  vHAC2 <- function(ld, lag_truncate) {
    vHAC(ld, k = lag_truncate, meth = "Bartlett")$hac
  }
  
  cex_gen <- 1.6
  P <- length(loss1)
  m <- round(mu * P)
  ld <- loss1 - loss2
  
  dm_final <- rep(NA, P - m + 1)
  
  if (!is.na(var(ld, na.rm = TRUE))){
    if(var(ld, na.rm = T) > 0){
      for (jj in m:P) {
        ind <- which(m:P == jj)
        ld_tmp <- ld[(jj - m + 1):jj]
        fit <- lm(ld_tmp ~ 1)
        mycov <- sandwich::NeweyWest(fit, lag = lag_truncate, prewhite = FALSE)
        fit <- lmtest::coeftest(fit, vcov = mycov)
        dm_final[ind]  <- fit[1,3]
      }
    }
  }
  
  
  
  if (is.null(time_labels)){
    time_labels <- 1:length(loss1)
  }
  inds <- floor(seq(from = m, to = length(time_labels), length.out = 5))
  df = data.frame(realtime_period = time_labels[m:length(loss1)], 
                  dmstat          = dm_final, 
                  window          = m)
  
  return(df)
}



# Newey-West VCV matrix
# u: vector of data
# prewhite: logical, should prewhitening be done?
# k: truncation lag for autocorrelations. If set to NULL, will be chosen automatically.
# meth: either "qs" (Quadratic Spectral, Andrews 1991) or anything else (Bartlett Kernel, Newey/West)
vHAC <- function(u, prewhite = FALSE, k = NULL, meth = "qs"){
  
  if (!is.matrix(u)) u <- as.matrix(u)
  
  n <- nrow(u)
  nreg <- ncol(u)
  rho <- sigma <- matrix(0, nreg, 1)
  
  # do a VAR(1) prewhitening
  if (prewhite == TRUE){
    reg.x <- matrix(u[1:(n-1), ], n-1, nreg)
    reg.y <- matrix(u[2:n, ], n-1, nreg)
    aux <- lm(reg.y~reg.x-1)
    beta <- matrix(unname(aux$coefficients), nreg, nreg)
    v <- matrix(unname(aux$residuals), n-1, nreg)
  } else {
    v <- u
    beta <- matrix(0, nreg, nreg)
  }
  nv <- nrow(v)
  
  # choose nr of lags (if not provided)
  if (is.null(k)){
    
    for (i in 1:nreg){
      aux <- lm(v[2:nv, i]~v[1:(nv-1), i]-1)
      rho[i] <- unname(aux$coefficients)
      sigma[i] <- sum(unname(aux$residuals)^2) / nv
    }
    
    if (meth == "qs"){
      # See Eq. (6.4) on page 835 of Andrews (1991) -> Note that his sigma^2 corresponds to our sigma
      top <- sum( (4*(rho^2) * (sigma^2)) / ((1-rho)^8) )
      bot <- sum( (sigma^2) / ((1-rho)^4) )
      k <- ceiling(1.3221*((top/bot)*n)^(0.2))    
    } else {
      top <- sum( (4*(rho^2) * (sigma^2)) / (((1-rho)^6)*((1+rho)^2)) )
      bot <- sum( (sigma^2) / ((1-rho)^4) )
      k <- min(c(ceiling(1.1447*((top/bot)*n)^(1/3)), round(0.5*n)))
    }
    
  }
  
  # compute HAC
  vcv <- (t(v) %*% v) / (n-1)
  
  if (k > 0){
    if (meth == "qs"){
      del <- ((6 * pi)/(5 * k)) * (1:(n-1))
      w <- 3 * (sin(del) / del - cos(del)) / (del^2)  
      if (prewhite == FALSE){
        mlag <- n - 1 
      } else {
        mlag <- nv - 1
      }
    } else {
      w <- 1 - (1:k)/(k+1)
      mlag <- k
    }
    for (i in 1:mlag){
      cov <- t(v[(i+1):nv, , drop = FALSE]) %*% (v[1:(nv-i), , drop = FALSE]) / (n-1)
      vcv <- vcv + w[i]*(cov + t(cov))
    }
  }
  
  d <- solve(diag(nreg) - t(beta))
  hac <- d %*% vcv %*% t(d)
  
  return(list(hac = hac, k = k))  
  
}

#########################3

make_plot_beta <- function(data)
{
  n.country = length(data)
  country.name = names(data)
  data      = lapply(1:n.country, function(x) mutate(data[[x]], Country = country.name[x]))
  in.gdp    = do.call(rbind, data)
  
  
  beta.gdp <- 
    in.gdp %>%
    mutate(Sentiment = factor(toupper(Sentiment), levels = toupper(sentx))) %>%
    ggplot(aes(horizon, Estimate)) + 
    geom_rect(data=data.frame(xmin = 30, xmax = 120, ymin=-Inf, ymax=Inf), 
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey97",  inherit.aes = FALSE)  +
    geom_vline(xintercept = seq(0, max(lag.max.q)+30, by = 30), color = "grey85",  linetype = "dashed") + 
    geom_vline(xintercept = seq(30, max(lag.max.q)+30, by = 90), color = "grey85") + 
    geom_line(color = "dodgerblue4", size = 0.8)  +
    geom_ribbon(aes(ymin=(Estimate - 1.64 * Std..Error),ymax=(Estimate + 1.64 * Std..Error)), alpha=0.3)+
    geom_hline(yintercept = 0, linetype = "dashed") + 
    theme_bw(base_size = 13) + 
    labs(x = "",y = "", title = toupper(dep)) +
    #coord_cartesian(xlim = c(-20, max(lag.max.q)+20),  expand = FALSE) +
    scale_x_reverse(breaks = c(0,seq(75, max(lag.max.q),by=90)), labels = c("rel.", "t", "t-1", "t-2","t-3","t-4")) + 
    #scale_x_reverse() +
    #scale_y_discrete(expand = c(0, 0)) + 
    theme(legend.position = "none", 
          axis.ticks = element_blank(), 
          axis.text.x = element_text(size = 10, angle = 0, hjust = 0.7),
          panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + 
    facet_grid(Sentiment ~ Country , scales = "free")
  
  return(beta.gdp)
}


make_plots_outsample <- function(res.gdp, dep, lag.max.q, lagDM, freq, start.forecast, 
                                 end.date, fluctuation = TRUE, my_mu = 10, country.i)
{
  
  test_bench <- "ARX"
  model.plot <- c("ARXS", "LASSO", "AVERAGE")
  model.plot <-  model.plot[!(model.plot %in% test_bench)]
  
  if (freq == "q")
  {
    hor.index  <- 30 ## sample the horizons in lag.max.q to start at this value (75 is mid of quarter t)
    hor.index1 <- which.min(abs(lag.max.q - hor.index)) -1
    #hor.plot   <-hor.plot <- lag.max.q[seq(3, length(lag.max.q)-8, length.out= 3)]
    hor.plot <- lag.max.q[seq(2, length(lag.max.q)-8, length.out= 4)]
    
    break.seq <- seq(90, max(lag.max.q),by=90)#[1:5]
    grey.area <- c(30, 120)
    vline     <- c(30, 30, 90)
    
  } else {
    
    break.seq <- seq(30, max(lag.max.q),by=30)[1:5]
    grey.area <- c(15,45)
    vline     <- c(15, 30, 30)
    
    hor.index <- 15 ## sample the horizons in lag.max.q to start at this value (75 is mid of quater t)
    hor.index <- which.min(abs(lag.max.q - hor.index)) 
    hor.plot  <- lag.max.q[seq(2, length(lag.max.q)-4, length.out= 3)]
  }
  
  #--------------------------------------------------#
  
  store.error <- res.gdp$forecast %>%
    mutate_at(vars(-c(realtime_period, horizon, Y)),
              funs(. - Y)) %>%
    left_join(nber, by = c("realtime_period" = "date")) 
  
  store.error <- store.error[!duplicated(store.error),]
  
  store.error.long <- res.gdp$forecast %>%
    gather(Model, Forecast, -realtime_period, -Y, -horizon) %>%
    mutate(Value = Forecast - Y,
           cycle = ifelse(Y < 0.75, "recession", "expansion")) 
  
  mpe <- store.error.long %>%
    #filter(!(realtime_period %in% c(as.Date("2009-06-05"), as.Date("2009-03-07")))) %>%
    group_by(Model, horizon) %>%
    summarize(MAPE = mean(abs(Value), na.rm=T),
              MSPE = mean(abs(Value)^2, na.rm=T)) %>%
    mutate(RMSPE = MSPE^(0.5)) %>%
    ungroup
  
  readr::write_csv(mpe, paste0(store.dir, "outofsample_prediction_error_", country.i, ".csv"))
  
  ## MSFE reduction across horizons
  library(RColorBrewer)
  myColors <- c("tomato4", rev(brewer.pal(9,"Blues"))[1:6])
  mySize <- c(4, rep(0.8,6))
  names(myColors) <- names(mySize) <- c("Benchmark", "ECONOMY", "FINSECTOR", "INFLATION", "MANUF", "MONPOL", "UNEMPLOYMENT")
  #c(sort(unique(mpe[substr(mpe$Model,1,3) == "ARX",]$Model)))
  
  
  msfe <- mpe %>%
    filter(substr(Model,1,3) == "ARX") %>%
    mutate(Model = case_when(Model == "ARX" ~ "Benchmark",
                             Model == "ARXN_economy" ~ "ECONOMY",
                             Model == "ARXN_finsector" ~ "FINSECTOR",
                             Model == "ARXN_inflation" ~ "INFLATION",
                             Model == "ARXN_indp" ~ "MANUF",
                             Model == "ARXN_monpol" ~ "MONPOL",
                             Model == "ARXN_unemployment" ~ "UNEMPLOYMENT")) %>%
    ggplot(aes(x=horizon, y=MSPE, color = Model)) +
    theme_light(base_size = 20) + 
    geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey85",  inherit.aes = FALSE)  +
    geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
    geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85") +
    geom_line(size = 0.8) +
    scale_color_jama() + 
    #scale_size_manual(values = mySize) +
    scale_x_reverse(breaks = c(0, break.seq),
                    labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) +
    theme(legend.position = "bottom", 
          legend.title = element_blank(),
          legend.text  = element_text(size = 16),
          panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank(),
          axis.text.x = element_text(angle=0)) +
    labs(x = NULL, y = NULL) 
  
  msfe_gray <- mpe %>%
    filter(substr(Model,1,3) == "ARX") %>%
    mutate(Model = case_when(Model == "ARX" ~ "Benchmark",
                             Model == "ARXN_economy" ~ "ECONOMY",
                             Model == "ARXN_finsector" ~ "FINSECTOR",
                             Model == "ARXN_inflation" ~ "INFLATION",
                             Model == "ARXN_indp" ~ "MANUF",
                             Model == "ARXN_monpol" ~ "MONPOL",
                             Model == "ARXN_unemployment" ~ "UNEMPLOYMENT")) %>%
    ggplot(aes(x=horizon, y=MSPE, color = Model)) +
    theme_light(base_size = 20) + 
    geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey85",  inherit.aes = FALSE)  +
    geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
    geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85") +
    geom_line(size = 0.8) +
    #scale_color_jama() + 
    #scale_size_manual(values = mySize) +
    scale_colour_manual(values = c("tomato4", rep("grey50",6))) +
    scale_x_reverse(breaks = c(0, break.seq),
                    labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) +
    theme(legend.position = "bottom", 
          legend.title = element_blank(),
          legend.text  = element_text(size = 16),
          panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank(),
          axis.text.x = element_text(angle=0)) +
    labs(x = NULL, y = NULL) 
  
  
  
  msfe_rel <- mpe %>%
    select(Model, horizon, MSPE) %>%
    spread(Model, MSPE) %>%
    select(horizon, contains("ARXN_"), average_arxn, ARX) %>%
    gather(Model, MSPE, -horizon, -ARX) %>%
    mutate(MSPE = MSPE / ARX) %>% 
    mutate(Model = case_when(Model == "ARX" ~ "Benchmark",
                             Model == "average_arxn" ~ "Average",
                             Model == "ARXN_economy" ~ "ECONOMY",
                             Model == "ARXN_finsector" ~ "FINSECTOR",
                             Model == "ARXN_inflation" ~ "INFLATION",
                             Model == "ARXN_indp" ~ "MANUF",
                             Model == "ARXN_monpol" ~ "MONPOL",
                             Model == "ARXN_unemployment" ~ "UNEMPLOYMENT"))
  
  msfe_rel_plot <- ggplot(msfe_rel, aes(x=horizon, y=MSPE, color=Model, linetype = Model)) +
    theme_light(base_size = 20) + 
    geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey85",  inherit.aes = FALSE)  +
    geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
    geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85") +
    geom_hline(yintercept = 1, color = "tomato4", linetype = "dashed", size = 1.5) + 
    geom_line() +
    geom_point(aes(shape = Model, size = Model), size=2) +     #geom_point(aes(shape = Model, size = Model)) + 
    #scale_size_manual(values = mySize) +
    scale_x_reverse(breaks = c(0, break.seq),
                    labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) +
    theme(legend.position = "bottom", 
          legend.title = element_blank(),
          legend.text  = element_text(size = 16),
          panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank(),
          axis.text.x = element_text(angle=0)) +
    labs(x = NULL, y = NULL) + ylim(0.97*min(msfe_rel$MSPE), 1.1*max(msfe_rel$MSPE)) +
    #scale_colour_manual(values = myColors)
    scale_color_jama()
  
  #save(msfe_rel, file="msfe_rel.Rdata")
  
  msfe_rel_gray <- ggplot(msfe_rel, aes(x=horizon, y=MSPE, color=Model)) +
    theme_light(base_size = 20) + 
    geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey85",  inherit.aes = FALSE)  +
    geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
    geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85") +
    geom_hline(yintercept = 1, color = "black", linetype = "dashed", size = 1.5) + 
    geom_line(size =1) +
    #scale_size_manual(values = mySize) +
    scale_x_reverse(breaks = c(0, break.seq),
                    labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) +
    theme(legend.position = "none", 
          legend.title = element_blank(),
          legend.text  = element_text(size = 16),
          panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank(),
          axis.text.x = element_text(angle=0)) +
    labs(x = NULL, y = NULL) + ylim(0.97*min(msfe_rel$MSPE), 1.1*max(msfe_rel$MSPE)) +
    scale_colour_manual(values = c("tomato4", rep("grey50",6))) + 
    #scale_color_jama() + 
    scale_y_continuous(breaks=seq(0.5,1.5,0.1))
  
  
  dm2.test_arx <- store.error %>% 
    select(-Y, contains("ARX"), average_arxn) %>%
    gather(Model, alternative, -realtime_period, -horizon, -!!test_bench)  %>%
    rename(null = !!test_bench) %>%
    group_by(Model, horizon) %>%
    do(myDM(.$null, .$alternative, lagDM, power = 2))  %>% ungroup 
  
  dm2cond.test_arx <- store.error %>% 
    select(realtime_period, horizon, nber, contains("ARX"), average_arxn) %>%
    gather(Model, alternative, -realtime_period, -horizon,  -nber, -!!test_bench)  %>%
    rename(null = !!test_bench) %>%
    group_by(Model, horizon) %>%
    do(myDMcond(.$null, .$alternative, .$nber, lagDM)[[1]])  %>% ungroup 
  
  dm1.test_arx <- store.error %>% 
    select(-Y) %>%
    gather(Model, alternative, -realtime_period, -horizon, -!!test_bench)  %>%
    rename(null = !!test_bench) %>%
    group_by(Model, horizon) %>%
    do(myDM(.$null, .$alternative, lagDM, power = 1))  %>% ungroup
  
  point.boxplot.dm2a <- 
    dm2.test_arx %>%
    #left_join(filter(dm2.test_arx, Model == "SS") %>% 
    #            select( horizon, DM) %>% 
    #            rename(LASSO = DM),
    #          by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_arxn") %>% 
                select(horizon, DM) %>% 
                rename(All = DM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_survey") %>% 
                select(horizon, DM) %>% 
                rename(Surveys = DM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_sent") %>% 
                select(horizon, DM) %>% 
                rename(Sentiments = DM),
              by= c("horizon")) %>%
    filter(substr(Model, 1, 4) == "ARXN") %>%
    mutate(Model = case_when(Model == "ARX" ~ "Benchmark",
                             Model == "average_arxn" ~ "Average",
                             Model == "ARXN_economy" ~ "ECONOMY",
                             Model == "ARXN_finsector" ~ "FINSECTOR",
                             Model == "ARXN_inflation" ~ "INFLATION",
                             Model == "ARXN_indp" ~ "MANUF",
                             Model == "ARXN_monpol" ~ "MONPOL",
                             Model == "ARXN_unemployment" ~ "UNEMPLOYMENT")) %>%
    ggplot(aes(horizon, DM, group = horizon)) + 
    #   geom_rect(data=data.frame(xmin = 15, xmax = 45, ymin=-Inf, ymax=Inf), 
    #              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
    #              fill="grey95",  inherit.aes = FALSE)  +
    #    geom_vline(xintercept = c(0,seq(15, max(lag.max.q), by = 30)), color = "grey85",  linetype = "dashed") + 
    #    geom_vline(xintercept = seq(15, max(lag.max.q), by = 30), color = "grey85") + 
    geom_boxplot(coef = 3, color = "gray70", width = 9, outlier.shape = NA) + 
    #geom_point(aes(horizon, LASSO, group = horizon), 
    #           shape = 0, size = 2, color = "seagreen4") +
    geom_point(aes(horizon, All, group = horizon), 
               shape = 5, size = 2, color = "dodgerblue1") +
    geom_point(aes(horizon, Surveys, group = horizon), 
               shape = 8, size = 2, color = "tomato1") +
    geom_point(aes(horizon, Sentiments, group = horizon), 
               shape = 1, size = 2, color = "cyan1") +
    geom_hline(yintercept = 0, color = "tomato3", linetype = "dashed") +
    # geom_hline(yintercept = 1.28, color = "tomato3") +
    geom_hline(yintercept = -1.28, color = "tomato3") +
    theme_light(base_size = 14) + 
    #   scale_x_reverse(breaks = c(0, break.seq), 
    #                    labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) + 
    theme(panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank()) + 
    labs(x = NULL, y = NULL) 
  
  point.boxplot.dm2b <- 
    dm2.test_arx %>%
    #left_join(filter(dm2.test_arx, Model == "SS") %>% 
    #            select( horizon, DM) %>% 
    #            rename(LASSO = DM),
    #          by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_arxn") %>% 
                select(horizon, DM) %>% 
                rename(All = DM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_survey") %>%
                select(horizon, DM) %>%
                rename(Surveys = DM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_sent") %>%
                select(horizon, DM) %>%
                rename(Sentiments = DM),
              by= c("horizon")) %>%
    filter(substr(Model, 1, 4) == "ARXN") %>%
    mutate(Model = case_when(Model == "ARX" ~ "Benchmark",
                             Model == "average_arxn" ~ "Average",
                             Model == "ARXN_economy" ~ "ECONOMY",
                             Model == "ARXN_finsector" ~ "FINSECTOR",
                             Model == "ARXN_inflation" ~ "INFLATION",
                             Model == "ARXN_indp" ~ "MANUF",
                             Model == "ARXN_monpol" ~ "MONPOL",
                             Model == "ARXN_unemployment" ~ "UNEMPLOYMENT")) %>%
    ggplot(aes(horizon, DM, group = horizon)) + 
    geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey85",  inherit.aes = FALSE)  +
    geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
    geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85") +
    geom_boxplot(coef = 3, color = "gray50", width = 9, outlier.shape = NA) + 
    #geom_point(aes(horizon, LASSO, group = horizon), shape = 0, size = 1.5, color = "seagreen4") +
    #geom_line(aes(horizon, LASSO, group = Model),  size = 0.25, color = "seagreen4") +
    geom_point(aes(horizon, All, group = horizon), shape = 5, size = 2, color = "dodgerblue4") +
    #geom_line(aes(horizon, All, group = Model), size = 0.25, color = "dodgerblue4") +
    #geom_point(aes(horizon, Surveys, group = horizon), shape = 8, size = 1.5, color = "tomato1") +
    #geom_line(aes(horizon, Surveys, group = Model),  size = 0.25, color = "tomato1") +
    #geom_point(aes(horizon, Sentiments, group = horizon), shape = 1, size = 1.5, color = "magenta3") +
    #geom_line(aes(horizon, Sentiments, group = Model),  size = 0.25, color = "magenta3") +
    geom_hline(yintercept = 0, color = "tomato3", linetype = "dashed") +
    #geom_hline(yintercept = 1.28, color = "tomato3") +
    geom_hline(yintercept = -1.28, color = "tomato3") +
    theme_light(base_size = 20) + 
    scale_x_reverse(breaks = c(0, break.seq),
                    labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) +
    theme(panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank()) + 
    labs(x = NULL, y = NULL) 
  
  point.boxplot.dm2c <- 
    dm2.test_arx %>%
    #left_join(filter(dm2.test_arx, Model == "SS") %>% 
    #            select( horizon, DM) %>% 
    #            rename(LASSO = DM),
    #          by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_arxn") %>% 
                select(horizon, pDM) %>% 
                rename(All = pDM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_survey") %>%
                select(horizon, DM) %>%
                rename(Surveys = DM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_sent") %>%
                select(horizon, DM) %>%
                rename(Sentiments = DM),
              by= c("horizon")) %>%
    filter(substr(Model, 1, 4) == "ARXN") %>%
    mutate(Model = case_when(Model == "ARX" ~ "Benchmark",
                             Model == "average_arxn" ~ "Average",
                             Model == "ARXN_economy" ~ "ECONOMY",
                             Model == "ARXN_finsector" ~ "FINSECTOR",
                             Model == "ARXN_inflation" ~ "INFLATION",
                             Model == "ARXN_indp" ~ "MANUF",
                             Model == "ARXN_monpol" ~ "MONPOL",
                             Model == "ARXN_unemployment" ~ "UNEMPLOYMENT")) %>%
    ggplot(aes(horizon, pDM, group = horizon)) + 
    geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey85",  inherit.aes = FALSE)  +
    geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
    geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85") +
    geom_boxplot(coef = 3, color = "gray50", width = 9, outlier.shape = NA) + 
    geom_point(aes(horizon, All, group = horizon), shape = 5, size = 2, color = "dodgerblue4") +
    geom_hline(yintercept = 0.10, color = "tomato3", linetype = "dashed") +
    geom_hline(yintercept = 0, color = "black") +
    geom_hline(yintercept = 1, color = "black") +
    theme_light(base_size = 20) + 
    scale_x_reverse(breaks = c(0, break.seq),
                    labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) +
    theme(panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank()) + 
    labs(x = NULL, y = NULL) 
  
  point.boxplot.dm2d <- 
    dm2.test_arx %>%
    #left_join(filter(dm2.test_arx, Model == "SS") %>% 
    #            select( horizon, DM) %>% 
    #            rename(LASSO = DM),
    #          by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_arxn") %>% 
                select(horizon, pDM) %>% 
                rename(All = pDM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_survey") %>%
                select(horizon, DM) %>%
                rename(Surveys = DM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_sent") %>%
                select(horizon, DM) %>%
                rename(Sentiments = DM),
              by= c("horizon")) %>%
    filter(substr(Model, 1, 4) == "ARXN" | Model == "average_arxn") %>%
    mutate(Model = case_when(Model == "ARX" ~ "Benchmark",
                             Model == "average_arxn" ~ "AVERAGE",
                             Model == "ARXN_economy" ~ "ECONOMY",
                             Model == "ARXN_finsector" ~ "FINSECTOR",
                             Model == "ARXN_inflation" ~ "INFLATION",
                             Model == "ARXN_indp" ~ "MANUF",
                             Model == "ARXN_monpol" ~ "MONPOL",
                             Model == "ARXN_unemployment" ~ "UNEMPLOYMENT")) %>%
    group_by(Model) %>%
    {
      ggplot(., aes(horizon, pDM, group = horizon)) + 
        geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
                  aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
                  fill="grey85",  inherit.aes = FALSE)  +
        geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
        geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85",size = 1) +
        #geom_vline(xintercept = horizon, color = "grey85", linetype = "dashed") +
        geom_point(aes(color = Model, group = Model, shape=Model),size = 2, alpha = 0.8) + 
        #geom_point(aes(color = Model, group = Model),size = 2, alpha = 0.4) +    
        geom_point(data = filter(., pDM < 0.101),   
                   aes(color = Model, group = Model, shape = Model), size =2) + #19) + 
        #geom_line(data = filter(., pDM < 0.101),   
        #         aes(color = Model, group = Model), size = 0.8, alpha = 0.8) +     
        geom_hline(yintercept = 0.10, color = "black") +
        geom_hline(yintercept = 0, color = "black", linetype = "dashed") +
        geom_hline(yintercept = 1, color = "black", linetype = "dashed") +
        theme_light(base_size = 20) + 
        scale_color_jama() + 
        scale_x_reverse(breaks = c(0, break.seq),
                        labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) +
        theme(panel.grid.major = element_blank(), 
              panel.grid.minor = element_blank(), 
              legend.position = "bottom",
              legend.title = element_blank()) + 
        labs(x = NULL, y = NULL) + 
        coord_cartesian(ylim = c(0, 1))    +
        scale_shape_manual(values=1:7) 
      
    }
  
  
  point.boxplot.dm2cond.p <- 
    dm2cond.test_arx %>%
    #left_join(filter(dm2.test_arx, Model == "SS") %>% 
    #            select( horizon, DM) %>% 
    #            rename(LASSO = DM),
    #          by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_arxn") %>% 
                select(horizon, pDM) %>% 
                rename(All = pDM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_survey") %>%
                select(horizon, DM) %>%
                rename(Surveys = DM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_sent") %>%
                select(horizon, DM) %>%
                rename(Sentiments = DM),
              by= c("horizon")) %>%
    filter(substr(Model, 1, 4) == "ARXN", MGW < 0.5) %>%
    mutate(Model = case_when(Model == "ARX" ~ "Benchmark",
                             Model == "average_arxn" ~ "Average",
                             Model == "ARXN_economy" ~ "ECONOMY",
                             Model == "ARXN_finsector" ~ "FINSECTOR",
                             Model == "ARXN_inflation" ~ "INFLATION",
                             Model == "ARXN_indp" ~ "MANUF",
                             Model == "ARXN_monpol" ~ "MONPOL",
                             Model == "ARXN_unemployment" ~ "UNEMPLOYMENT")) %>%
    ggplot(aes(horizon, pDM, group = horizon)) + 
    geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey85",  inherit.aes = FALSE)  +
    geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
    geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85") +
    geom_boxplot(coef = 3, color = "gray50", width = 9, outlier.shape = NA) + 
    geom_point(aes(horizon, All, group = horizon), shape = 5, size = 2, color = "dodgerblue4") +
    geom_hline(yintercept = 0.10, color = "tomato3", linetype = "dashed") +
    geom_hline(yintercept = 0, color = "black") +
    geom_hline(yintercept = 1, color = "black") +
    theme_light(base_size = 20) + 
    scale_x_reverse(breaks = c(0, break.seq),
                    labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) +
    theme(panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank()) + 
    labs(x = NULL, y = NULL) 
  
  point.boxplot.dm2cond.igw <- 
    dm2cond.test_arx %>%
    #left_join(filter(dm2.test_arx, Model == "SS") %>% 
    #            select( horizon, DM) %>% 
    #            rename(LASSO = DM),
    #          by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_arxn") %>% 
                select(horizon, pDM) %>% 
                rename(All = pDM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_survey") %>%
                select(horizon, DM) %>%
                rename(Surveys = DM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_sent") %>%
                select(horizon, DM) %>%
                rename(Sentiments = DM),
              by= c("horizon")) %>%
    filter(substr(Model, 1, 4) == "ARXN") %>%
    mutate(Model = case_when(Model == "ARX" ~ "Benchmark",
                             Model == "average_arxn" ~ "Average",
                             Model == "ARXN_economy" ~ "ECONOMY",
                             Model == "ARXN_finsector" ~ "FINSECTOR",
                             Model == "ARXN_inflation" ~ "INFLATION",
                             Model == "ARXN_indp" ~ "MANUF",
                             Model == "ARXN_monpol" ~ "MONPOL",
                             Model == "ARXN_unemployment" ~ "UNEMPLOYMENT")) %>%
    ggplot(aes(horizon, IGW, group = horizon)) + 
    geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey85",  inherit.aes = FALSE)  +
    geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
    geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85") +
    geom_boxplot(coef = 3, color = "gray50", width = 9, outlier.shape = NA) + 
    #geom_point(aes(horizon, All, group = horizon), shape = 5, size = 2, color = "dodgerblue4") +
    geom_hline(yintercept = 0.10, color = "tomato3", linetype = "dashed") +
    geom_hline(yintercept = 0, color = "black") +
    geom_hline(yintercept = 1, color = "black") +
    theme_light(base_size = 20) + 
    scale_x_reverse(breaks = c(0, break.seq),
                    labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) +
    theme(panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank()) + 
    labs(x = NULL, y = NULL) 
  
  point.boxplot.dm2cond.mgw <- 
    dm2cond.test_arx %>%
    #left_join(filter(dm2.test_arx, Model == "SS") %>% 
    #            select( horizon, DM) %>% 
    #            rename(LASSO = DM),
    #          by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_arxn") %>% 
                select(horizon, pDM) %>% 
                rename(All = pDM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_survey") %>%
                select(horizon, DM) %>%
                rename(Surveys = DM),
              by= c("horizon")) %>%
    left_join(filter(dm2.test_arx, Model == "average_sent") %>%
                select(horizon, DM) %>%
                rename(Sentiments = DM),
              by= c("horizon")) %>%
    filter(substr(Model, 1, 4) == "ARXN") %>%
    mutate(Model = case_when(Model == "ARX" ~ "Benchmark",
                             Model == "average_arxn" ~ "Average",
                             Model == "ARXN_economy" ~ "ECONOMY",
                             Model == "ARXN_finsector" ~ "FINSECTOR",
                             Model == "ARXN_inflation" ~ "INFLATION",
                             Model == "ARXN_indp" ~ "MANUF",
                             Model == "ARXN_monpol" ~ "MONPOL",
                             Model == "ARXN_unemployment" ~ "UNEMPLOYMENT")) %>%
    ggplot(aes(horizon, MGW, group = horizon)) + 
    geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey85",  inherit.aes = FALSE)  +
    geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
    geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85") +
    geom_boxplot(coef = 3, color = "gray50", width = 9, outlier.shape = NA) + 
    #geom_point(aes(horizon, All, group = horizon), shape = 5, size = 2, color = "dodgerblue4") +
    geom_hline(yintercept = 0.10, color = "tomato3", linetype = "dashed") +
    geom_hline(yintercept = 0, color = "black") +
    geom_hline(yintercept = 1, color = "black") +
    theme_light(base_size = 20) + 
    scale_x_reverse(breaks = c(0, break.seq),
                    labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) +
    theme(panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank()) + 
    labs(x = NULL, y = NULL) 
  
  point.boxplot.dm2dcond <- 
    dm2cond.test_arx %>%
    filter(substr(Model, 1, 4) == "ARXN" | Model == "average_arxn", MGW < 0.5) %>%
    mutate(Model = case_when(Model == "ARX" ~ "Benchmark",
                             Model == "average_arxn" ~ "AVERAGE",
                             Model == "ARXN_economy" ~ "ECONOMY",
                             Model == "ARXN_finsector" ~ "FINSECTOR",
                             Model == "ARXN_inflation" ~ "INFLATION",
                             Model == "ARXN_indp" ~ "MANUF",
                             Model == "ARXN_monpol" ~ "MONPOL",
                             Model == "ARXN_unemployment" ~ "UNEMPLOYMENT")) %>%
    group_by(Model) %>%
    {
      ggplot(., aes(horizon, pDM, color = Model, group = Model)) + 
        geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
                  aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
                  fill="grey85",  inherit.aes = FALSE)  +
        geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
        geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85",size = 1) +
        #geom_vline(xintercept = horizon, color = "grey85", linetype = "dashed") +
        geom_hline(yintercept = 0.10, color = "black", linetype = "dashed") +
        geom_hline(yintercept = 0, color = "black", linetype = "dashed") +
        geom_hline(yintercept = 1, color = "black", linetype = "dashed") +
        geom_point(aes(color = Model, group = Model, shape=Model),size = 2, alpha = 0.8) + 
        #geom_point(aes(color = Model, group = Model),size = 2, alpha = 0.4) +   
        #geom_line(data = filter(., pDM < 0.101), aes(color = Model, group = Model), size = 0.8) + 
        #geom_point(data = filter(., pDM < 0.101), aes(color = Model, group = Model), size = 2) + 
        geom_point(data = filter(., pDM < 0.101),   
                   aes(color = Model, group = Model, shape=Model), size =2) + #, shape = 19) + 
        geom_hline(yintercept = 0.10, color = "black") +
        scale_color_jama() + 
        theme_light(base_size = 20) + 
        scale_x_reverse(breaks = c(0, break.seq),
                        labels = c("rel.", "t", "t-1", "t-2", "t-3", "t-4")) +
        theme(panel.grid.major = element_blank(), 
              panel.grid.minor = element_blank(),
              legend.position = "bottom", legend.title = element_blank()) + 
        labs(x = NULL, y = NULL)  + 
        coord_cartesian(ylim = c(0, 1)) +
        scale_shape_manual(values=1:7) 
    }
  
  
  
  if (fluctuation)
  {
    cv_fluct <- data.frame(mu = as.integer(seq(from = 10, to = 90, by = 10)), 
                           twosid_5  = c(3.393, 3.179, 3.012, 2.89, 2.779, 2.634, 2.56, 2.433, 2.248), 
                           twosid_10 = c(3.17, 2.948, 2.766, 2.626, 2.5, 2.356, 2.252, 2.13, 1.95),
                           onesid_5  = c(3.176, 2.938, 2.770, 2.624, 2.475, 2.352, 2.248, 2.080, 1.975),
                           onesid_10 = c(2.928, 2.676, 2.482, 2.334, 2.168, 2.030, 1.904, 1.740, 1.600))
    
    fluct_test <- 
      store.error %>%
      filter(horizon %in% hor.plot) %>%
      select(-Y) %>%
      mutate_at(vars(-c(realtime_period, horizon)), funs(abs(.)^2)) %>%
      gather(Model, Alternative, -!!test_bench, -realtime_period, -horizon) %>%
      group_by(horizon, Model) %>%
      #mutate_at(vars(-c(realtime_period, horizon, ARX)), funs(fluctuation_test(ARX, ., time_labels = realtime_period, mu=0.1))) 
      do(myfluctuation_test(pull(., Alternative), 
                            pull(., !!test_bench), 
                            mu           = (my_mu/100),
                            lag_truncate = lagDM, 
                            time_labels  = pull(., realtime_period),
                            conf_level   = 0.10)) # %>% filter(realtime_period < as.Date("2019-11-01")) 
    
    a <- fluct_test %>% group_by(realtime_period, horizon) %>% count %>% arrange(desc(realtime_period))
    a %>% arrange(desc(n))
    table(a$n)
    
    if(freq == "q") fluct_test <- mutate(fluct_test, mid_window  = realtime_period %m+% months(-round(3*window/2)))
    if(freq == "m") fluct_test <- mutate(fluct_test, mid_window  = realtime_period %m+% months(-round(window/2)))
    
    sent0f <- fluct_test %>% ungroup() %>%
      filter(grepl("ARXN", Model)) %>%
      select(Model) %>%
      unique(.) 
    
    
    sent1f  <- data.frame(sent0 = sent0f$Model, sent1 = substr(sent0f$Model, 6, nchar(sent0f$Model)), stringsAsFactors = FALSE) %>%
      mutate(sent1 = ifelse(substr(sent1, 1, 4) %in% c("cum_","av_"), substr(sent1, 5,nchar(sent1)), sent1)) %>%
      separate(sent1, into = c("term", "tense"), sep = "_") %>%
      mutate(tense = plyr::mapvalues(tense,
                                     from = c("pa1","pr1","fu1","nan2","paprfunan1"),
                                     to = c("past","present","future","nan","")),
             group = ifelse(term %in% sentx2, "SURVEY","SENTIMENT"),
             term = plyr::mapvalues(term, from = c("indp","monpol"), to = c("manufacturing","monetary policy")),
             term = toupper(term),
             sent1 = ifelse(is.na(tense), term, paste0(term, tense))) %>% 
      select(-term, -tense)
    
    # NEW: HERE I SELECT THE MODELS THAT SIGNIFICANTLY IMPROVE AND PLOT ONLY THOSE ... TO INCREASE READABILITY
    fluct.select <- fluct_test %>% 
      filter(horizon %in% hor.plot, substr(Model, 1, 5) == "ARXN_") %>%
      group_by(Model, horizon) %>% 
      summarize(fmin = min(dmstat, na.rm = T)) %>% 
      ungroup() %>%
      #filter(fmin < -2.7) %>%
      select(Model, horizon)
    fluct.select <- rbind(fluct.select, data.frame(Model = "average_arxn", horizon = 45))
    
    fluct.min <- min((filter(ungroup(fluct_test), horizon %in% hor.plot, Model %in% fluct.select$Model) %>% select(dmstat) %>% min(.)), -1.05 * cv_fluct[which(cv_fluct$mu == my_mu),"onesid_10"])
    fluct.max <- max((filter(ungroup(fluct_test), horizon %in% hor.plot, Model %in% fluct.select$Model) %>% select(dmstat) %>% max(.)), 1.05 * cv_fluct[which(cv_fluct$mu == my_mu),"onesid_10"])
    #fluct.caption <- paste0("Rolling window of ", window, ifelse(freq == "m", " months"," quarters"))
    
    fluct.gdp <- 
      fluct_test %>% 
      ungroup %>%
      rename(Sentiment = Model) %>%
      filter(Sentiment %in% fluct.select$Model) %>% 
      mutate(Sentiment = plyr::mapvalues(Sentiment, from = c(sent1f$sent0,"average_arxn"), to = c(sent1f$sent1,"AVERAGE")),
             Group     = plyr::mapvalues(Sentiment, from = c(sent1f$sent1,"average_arxn"), to = c(sent1f$group,"AVERAGE"))) %>%  
      mutate(horizon = factor(paste(round((horizon - 1)/7), "weeks"), levels = unique(paste(round((horizon - 1)/7), "weeks")), ordered = TRUE)) %>%
      ggplot() +
      geom_rect(data=(data.frame(xmin=cepr.min, xmax=cepr.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
                aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
                fill="gray80",alpha=0.3) +
      geom_line(aes(mid_window, dmstat, color = Sentiment, group = Sentiment)) + 
      scale_color_jama() +
      #geom_hline(yintercept = cv_fluct[which(cv_fluct$mu == my_mu),"onesid_10"], color = "black", linetype = "dashed") + 
      geom_hline(yintercept = -cv_fluct[which(cv_fluct$mu == my_mu),"onesid_10"], color = "black", linetype = "dashed") +  
      geom_hline(yintercept = 0, color = "black", linetype = "dotted") +
      theme_bw(base_size = 20) + 
      xlim(as.Date(start.forecast), as.Date(end.date)) + 
      #ylim(max(fluct.min, -5), min(fluct.max,5)) +   
      #coord_cartesian(ylim=c(max(fluct.min, -6), min(fluct.max,6))) + 
      coord_cartesian(ylim=c(-6, 6)) + 
      labs(x = NULL, y = NULL, title = dep) + 
      theme(legend.position = "bottom", 
            legend.title = element_blank(),
            legend.text  = element_text(size = 16),
            panel.grid.major = element_blank(), 
            panel.grid.minor = element_blank(),
            axis.text.x = element_text(angle=0)) +
      facet_wrap(~horizon)
    
    
    fluct.gdp.gray <- 
      fluct_test %>% 
      ungroup %>%
      rename(Sentiment = Model) %>%
      filter(Sentiment %in% fluct.select$Model) %>% 
      mutate(Sentiment = plyr::mapvalues(Sentiment, from = c(sent1f$sent0,"average_arxn"), to = c(sent1f$sent1,"AVERAGE")),
             Group     = plyr::mapvalues(Sentiment, from = c(sent1f$sent1,"average_arxn"), to = c(sent1f$group,"AVERAGE"))) %>%  
      mutate(horizon = factor(paste(round((horizon - 1)/7), "weeks"), levels = unique(paste(round((horizon - 1)/7), "weeks")), ordered = TRUE)) %>%
      ggplot() +
      geom_rect(data=(data.frame(xmin=cepr.min, xmax=cepr.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
                aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
                fill="gray70",alpha=0.3) +
      geom_line(aes(mid_window, dmstat, color = Sentiment, size = Sentiment, linetype = Sentiment)) + 
      #scale_color_jama() +
      #geom_hline(yintercept = cv_fluct[which(cv_fluct$mu == my_mu),"onesid_10"], color = "black", linetype = "dashed") + 
      geom_hline(yintercept = -cv_fluct[which(cv_fluct$mu == my_mu),"onesid_10"], color = "black", linetype = "dashed") +  
      geom_hline(yintercept = 0, color = "black", linetype = "dotted") +
      theme_bw(base_size = 20) + 
      xlim(as.Date(start.forecast), as.Date(end.date)) + 
      ylim(fluct.min, fluct.max) +   
      labs(x = NULL, y = NULL, title = dep) + 
      theme(legend.position = "none", 
            legend.title = element_blank(),
            legend.text  = element_text(size = 16),
            panel.grid.major = element_blank(), 
            panel.grid.minor = element_blank(),
            axis.text.x = element_text(angle=0)) +
      scale_colour_manual(values = c("tomato4", rep("grey50",6))) + 
      scale_size_manual(values = c(1.5, rep(0.8,6))) +
      scale_linetype_manual(values = c("solid", rep("F1",6))) +
      facet_wrap(~horizon)
    
    fluct.gdp.average <- 
      fluct_test %>% ungroup %>%
      rename(Sentiment = Model) %>%
      filter(horizon %in% hor.plot, Sentiment %in% c("average_survey", "average_sent", "average_arxn")) %>% 
      mutate(Sentiment = plyr::mapvalues(Sentiment, 
                                         from = c("average_survey", "average_sent", "average_arxn"), 
                                         to = c("AVERAGE SURVEY", "AVERAGE SENTIMENT", "AVERAGE ALL"))) %>%  
      mutate(horizon = factor(paste(round((horizon - 1)/7), "weeks"), 
                              levels = unique(paste(round((horizon - 1)/7), "weeks")), 
                              ordered = TRUE)) %>%
      ggplot() +
      geom_rect(data=(data.frame(xmin=cepr.min, xmax=cepr.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
                aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
                fill="gray70",alpha=0.3) +
      geom_line(aes(mid_window, dmstat, color = Sentiment, group = Sentiment)) + 
      scale_color_jama() +
      geom_hline(yintercept = cv_fluct[which(cv_fluct$mu == my_mu),"onesid_10"], color = "black", linetype = "dashed") + 
      geom_hline(yintercept = -cv_fluct[which(cv_fluct$mu == my_mu),"onesid_10"], color = "black", linetype = "dashed") +  
      geom_hline(yintercept = 0, color = "black", linetype = "dotted") +
      theme_bw(base_size = 20) + 
      xlim(as.Date(start.forecast), as.Date(end.date)) + 
      ylim(fluct.min, fluct.max) +   
      labs(x = NULL, y = NULL, title = dep) + 
      theme(legend.position = "none", 
            panel.grid.major = element_blank(),
            strip.text = element_text(size = 18),
            legend.title = element_blank(),
            legend.text  = element_text(size = 18),
            panel.grid.minor = element_blank(),
            axis.text.x = element_text(angle=0)) +
      facet_wrap(~horizon)
    
    
  } else {fluct.gdp <- NULL; fluct.gdp.average <- NULL}
  
  
  plots <- list(point.boxplot.dm2a, point.boxplot.dm2b, 
                fluct.gdp, fluct.gdp.gray, fluct.gdp.average, 
                msfe, msfe_rel_plot, msfe_rel_gray,
                point.boxplot.dm2c, point.boxplot.dm2cond.p,
                point.boxplot.dm2d,point.boxplot.dm2dcond)
  
  return(plots)
}


make_plots_covid <- function(res.gdp, dep, lag.max.q, lagDM, freq = "q", fluctuation = TRUE)
{
  
  test_bench <- "ARX"
  model.plot <- c("ARXS", "AVERAGE")
  model.plot <-  model.plot[!(model.plot %in% test_bench)]
  
  if (freq == "q")
  {
    hor.index  <- 60 ## sample the horizons in lag.max.q to start at this value (75 is mid of quater t)
    hor.index1 <- which.min(abs(lag.max.q - hor.index)) -1
    hor.plot   <- lag.max.q[seq(hor.index1, length(lag.max.q), by= 6)]
    
    break.seq <- seq(75, max(lag.max.q)+30,by=90)[1:5]
    grey.area <- c(45, 135)
    vline     <- c(15, 30, 90)
    
  } else {
    
    break.seq <- seq(30, max(lag.max.q),by=30)[1:5]
    grey.area <- c(15,45)
    vline     <- c(15, 30, 30)
    
    hor.index <- 15 ## sample the horizons in lag.max.q to start at this value (75 is mid of quater t)
    hor.index <- which.min(abs(lag.max.q - hor.index)) 
    hor.plot  <- lag.max.q[seq(hor.index, length(lag.max.q), by= 4)]
  }
  
  #--------------------------------------------------#
  
  ## plot forecasts
  
  p.ave <- 
    res.gdp$forecast %>%
    mutate(horizon = factor(paste(horizon, "days"), levels = unique(paste(horizon, "days")), ordered = TRUE)) %>%
    select(realtime_period, horizon, Y, average) %>%
    ggplot() +
    geom_rect(data=(data.frame(xmin=cepr.min, xmax=cepr.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="gray80",alpha=0.3) +
    geom_point(aes(realtime_period, average, group = factor(horizon), color = factor(horizon)), size = 1) +
    #geom_line(aes(realtime_period, Y), color= "gray50") +
    geom_hline(yintercept = 0, linetype = "dotted") + 
    theme_light() + theme(legend.position = "bottom", panel.grid.major = element_blank(), panel.grid.minor = element_blank())  + 
    guides(colour = guide_legend(nrow = 2, title="Horizon", label.theme = element_text(size = 8))) +
    scale_color_brewer(palette="RdYlBu") +
    labs(x = NULL, y = NULL, caption = "Average forecasts") + 
    ggtitle(dep)
  
  # p.bench1 <- 
  #   res.gdp$forecast %>%
  #   mutate(horizon = factor(paste(horizon, "days"), levels = unique(paste(horizon, "days")), ordered = TRUE)) %>%
  #   select(realtime_period, horizon, Y, BENCH1) %>%
  #   ggplot() +
  #   geom_rect(data=(data.frame(xmin=cepr.min, xmax=cepr.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
  #             aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
  #             fill="gray80",alpha=0.3) +
  #   geom_point(aes(realtime_period, BENCH1, group = factor(horizon), color = factor(horizon)), size = 1) +
  #   #geom_line(aes(realtime_period, Y), color= "gray50") +
  #   geom_hline(yintercept = 0, linetype = "dotted") + 
  #   theme_light() + theme(legend.position = "bottom", panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + 
  #   guides(colour = guide_legend(nrow = 2, title="Horizon", label.theme = element_text(size = 8))) + 
  #   scale_color_brewer(palette="RdYlBu") +
  #   labs(x = NULL, y = NULL, caption = paste("Model:",benchmark1)) + 
  #   ggtitle(dep)
  # 
  # p.bench2 <- 
  #   res.gdp$forecast %>%
  #   mutate(horizon = factor(paste(horizon, "days"), levels = unique(paste(horizon, "days")), ordered = TRUE)) %>%
  #   select(realtime_period, horizon, Y, BENCH2) %>%
  #   ggplot() +
  #   geom_rect(data=(data.frame(xmin=cepr.min, xmax=cepr.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
  #             aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
  #             fill="gray80",alpha=0.3) +
  #   geom_point(aes(realtime_period, BENCH2, group = factor(horizon), color = factor(horizon)), size = 1) +
  #   #geom_line(aes(realtime_period, Y), color= "gray50") +
  #   geom_hline(yintercept = 0, linetype = "dotted") + 
  #   theme_light() + theme(legend.position = "bottom", panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + 
  #   guides(colour = guide_legend(nrow = 2, title="Horizon", label.theme = element_text(size = 8))) + 
  #   scale_color_brewer(palette="RdYlBu") +
  #   labs(x = NULL, y = NULL, caption = paste("Model:",benchmark2)) + 
  #   ggtitle(dep)
  
  p.arx <- 
    res.gdp$forecast %>%
    mutate(horizon = factor(paste(horizon, "days"), levels = unique(paste(horizon, "days")), ordered = TRUE)) %>%
    select(realtime_period, horizon, Y, ARX) %>%
    ggplot() +
    geom_rect(data=(data.frame(xmin=cepr.min, xmax=cepr.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="gray80",alpha=0.3) +
    geom_point(aes(realtime_period, ARX, group = factor(horizon), color = factor(horizon)), size = 1) +
    #geom_line(aes(realtime_period, Y), color= "gray50") +
    geom_hline(yintercept = 0, linetype = "dotted") + 
    theme_light() + theme(legend.position = "bottom", panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + 
    guides(colour = guide_legend(nrow = 2, title="Horizon", label.theme = element_text(size = 8))) +
    scale_color_brewer(palette="RdYlBu") +
    labs(x = NULL, y = NULL, caption = "ARX forecasts") + 
    ggtitle(dep)
  
  p.arxn <- 
    res.gdp$forecast %>%
    mutate(horizon = factor(paste(horizon, "days"), 
                            levels = unique(paste(horizon, "days")), 
                            ordered = TRUE),
           reference_quarter = realtime_period - 90,
           reference_quarter = paste0(year(reference_quarter),
                                      "Q", 
                                      quarter(reference_quarter))) %>%
    select(reference_quarter, horizon, Y, contains("ARXN_"), ARX) %>%
    gather(Model, Forecast, -reference_quarter, -horizon, -Y, -ARX) %>%
    #filter(horizon %in% c("14 days", "42 days", "70 days")) %>%
    {
      ggplot(.) +
        geom_boxplot(aes(reference_quarter, Forecast, fill = factor(horizon)), 
                     size = 0.1, 
                     outlier.shape = NA,
                     outlier.size = 5, 
                     color = "black") +
        geom_point(data = filter(., Model == "index"), aes(reference_quarter, Forecast), color="black", shape = 12) +
        geom_text(aes( x=factor(reference_quarter), 
                       y=1.3*min(select(res.gdp$forecast,contains("ARXN_")), na.rm=T), 
                       label=round(Y,2)),                 , 
                  color="gray48", 
                  size= 6, fontface="plain") +
        #size= 5,  fontface = "italic") +
        geom_hline(yintercept = 0, linetype = "dotted") + 
        labs(x = NULL, y = NULL, title = NULL)  +
        scale_fill_brewer(palette="RdYlBu") +
        #scale_color_brewer(palette="RdBu") +
        guides(fill = guide_legend(nrow = 1, 
                                   title="Horizon",
                                   label.theme = element_text(size = 18)),
               color = FALSE) +
        theme_light(base_size = 18) + 
        #scale_x_discrete(limits=factor(res.gdp$forecast$realtime_period),
        #                 labels=paste0(year(res.gdp$forecast$realtime_period),
        #                               "Q", 
        #                               quarter(res.gdp$forecast$realtime_period))) + 
        theme(legend.position = "bottom", 
              panel.grid.major = element_blank(), 
              panel.grid.minor = element_blank(),
              axis.text.x = element_text(angle = 0, vjust = 0.5, size = 16),
              legend.key = element_rect(fill = NA, color = NA)) 
    }
  
  p.arxn1 <- 
    res.gdp$forecast %>%
    mutate(horizon = factor(paste(horizon, "days"), levels = unique(paste(horizon, "days")), ordered = TRUE)) %>%
    select(realtime_period, horizon, Y, contains("ARXN_"), ARX) %>%
    gather(Model, Forecast, -realtime_period, -horizon, -Y, -ARX) %>%
    #filter(horizon %in% c("14 days", "42 days", "70 days")) %>%
    ggplot() +
    geom_boxplot(aes(factor(realtime_period), Forecast, fill = factor(horizon)), 
                 size = 0.1, 
                 outlier.shape = NA,
                 color = "black") +
    geom_point(data = res.gdp$forecast, aes(factor(realtime_period), ARX, color= factor(horizon)), shape = 8, size=1) +
    #annotate("text", x = factor(res.gdp$forecastrealtime_period), y = rep(-4, nrow(res.gdp$forecast)), label = round(res.gdp$forecast$Y,2)) +
    geom_text(aes(x=factor(realtime_period),
                  y=1.2*min(select(res.gdp$forecast,contains("ARXN_")), na.rm=T), 
                  label=round(Y,2)),                 , 
              color="black", size= 3, family = "Times", fontface = "italic") +
    geom_hline(yintercept = 0, linetype = "dotted") + 
    labs(x = NULL, y = NULL, title = NULL)  +
    scale_fill_brewer(palette="RdYlBu") +
    #scale_color_brewer(palette="RdBu") +
    guides(fill = guide_legend(nrow = 1, 
                               title="Horizon", 
                               label.theme = element_text(size = 12)),
           color = FALSE) +
    theme_light() + 
    scale_x_discrete(limits=factor(res.gdp$forecast$realtime_period),
                     labels=paste0(year(res.gdp$forecast$realtime_period),
                                   "Q", 
                                   quarter(res.gdp$forecast$realtime_period))) + 
    theme(legend.position = "bottom", 
          panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank(),
          text = element_text(size=16),
          axis.text.x = element_text(angle=15, size=12),
          legend.key = element_rect(fill = NA, color = NA)) 
  
  p.ar <- 
    res.gdp$forecast %>%
    mutate(horizon = factor(paste(horizon, "days"), levels = unique(paste(horizon, "days")), ordered = TRUE)) %>%
    select(realtime_period, horizon, Y, AR) %>%
    ggplot() +
    geom_rect(data=(data.frame(xmin=cepr.min, xmax=cepr.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="gray80",alpha=0.3) +
    geom_point(aes(realtime_period, AR, group = factor(horizon), color = factor(horizon)), size = 1) +
    #geom_line(aes(realtime_period, Y), color= "gray50") +
    geom_hline(yintercept = 0, linetype = "dotted") + 
    theme_light() + theme(legend.position = "bottom", panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + 
    guides(colour = guide_legend(nrow = 2, title="Horizon", label.theme = element_text(size = 8))) +
    scale_color_brewer(palette="RdYlBu") +
    labs(x = NULL, y = NULL, caption = "AR forecasts") + 
    ggtitle(dep)
  
  p.ss <- 
    res.gdp$forecast %>%
    mutate(horizon = factor(paste(horizon, "days"), levels = unique(paste(horizon, "days")), ordered = TRUE)) %>%
    select(realtime_period, horizon, Y, SS) %>%
    ggplot() +
    geom_rect(data=(data.frame(xmin=cepr.min, xmax=cepr.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="gray80",alpha=0.3) +
    geom_point(aes(realtime_period, SS, group = factor(horizon), color = factor(horizon)), size = 1) +
    #geom_line(aes(realtime_period, Y), color= "gray50") +
    geom_hline(yintercept = 0, linetype = "dotted") + 
    theme_light() + theme(legend.position = "bottom", panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
    guides(colour = guide_legend(nrow = 2, title="Horizon", label.theme = element_text(size = 8))) + 
    scale_color_brewer(palette="RdYlBu") +
    labs(x = NULL, y = NULL, caption = "LASSO") + 
    ggtitle(dep)
  
  # p.news <- 
  #   res.gdp$forecast %>%
  #   filter(horizon %in% hor.plot) %>%
  #   mutate(horizon = factor(paste(horizon, "days"), levels = unique(paste(horizon, "days")), ordered = TRUE)) %>%
  #   select(realtime_period, horizon, Y, NEWS) %>%
  #   ggplot() +
  #   geom_rect(data=(data.frame(xmin=cepr.min, xmax=cepr.max, ymin=-Inf, ymax=Inf) %>% filter(xmin > start.forecast)),
  #             aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
  #             fill="gray80",alpha=0.3) +
  #   geom_point(aes(realtime_period, NEWS, group = factor(horizon), color = factor(horizon)), size = 1) +
  #   geom_line(aes(realtime_period, Y), color= "gray50") +
  #   geom_hline(yintercept = 0, linetype = "dotted") + 
  #   theme_light() + theme(legend.position = "bottom", panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
  #   guides(colour = guide_legend(nrow = 2, title="Horizon", label.theme = element_text(size = 8))) + 
  #   scale_color_brewer(palette="RdBu") +
  #   labs(x = NULL, y = NULL) + 
  #   ggtitle(dep)
  
  
  ts.gdp <- ggpubr::ggarrange(p.arx + ggtitle("ARX"), 
                              p.ar + ggtitle("AR"),  
                              p.ave + ggtitle("Average"), 
                              p.ss + ggtitle("LASSO"), ncol = 2, nrow = 2, common.legend = TRUE, legend = "bottom")
  #ts.gdp <- list(ts.gdp, p.arx, p.ar, p.ave, p.ss, p.bench1, p.bench2, p.arxn, p.arxn1)
  ts.gdp <- list(ts.gdp, p.arx, p.ar, p.ave, p.ss, p.arxn, p.arxn1)
  
  plots <- list(ts.gdp)
  
  return(plots)
}

causal_data <-function(country.i, 
                       mix.freq, 
                       type = "av", 
                       final = "2019-12-31", 
                       fun = "mean",
                       mymu_fluct = 10,
                       symb.list_all,
                       store.dir
                       
)
{
  
  if (type == "av")
  {
    sentiment <- DNA_sentiment_av %>%
      filter(country == country.i, measure == type) %>%
      select(realtime_period, topic, value) %>%
      spread(topic, value) %>%
      mutate(average = (economy+finsector+indp+inflation+monpol+unemployment)/6)
  }
  if (type == "cum")
  {
    sentiment <- DNA_sentiment_cum %>%
      filter(country == country.i, measure == type) %>%
      select(realtime_period, topic, value) %>%
      spread(topic, value) %>%
      mutate(average = (economy+finsector+indp+inflation+monpol+unemployment)/6)
  }
  
  sent.names <- names(sentiment)[-1]
  
  ############################################################################
  
  df.sentiment <- roll_sentiment_US(sentiment, roll_type,  lambda, roll.window.sent1, fun) 
  
  macro <- get_macrodata(country    = country.i, 
                         where      = "alfred", 
                         start.date = start.date, 
                         end.date   = "2021-01-01", 
                         vintage    = vintage,
                         save.it    = save.it, 
                         folder.fred = macro.dir, 
                         symb.list_all) 
  
  values_from   <- filter(symb.list_all, country == country.i) %>% select(symb) %>% unlist %>% c()
  values_to     <- filter(symb.list_all, country == country.i) %>% select(Description) %>% unlist %>% c()
  
  myvar     <- c("IND", "CPI","UNE")
  index       <- which(substr(values_to, 1, 3) %in% myvar)
  for (i in 1:3) values_from   <- c(values_from, paste0(values_from[index], "_",i))
  for (i in 1:3) values_to     <- c(values_to, paste0(values_to[index], "_",i))
  
  macro[[1]] <- 
    macro[[1]] %>% 
    mutate(Symbol = plyr::mapvalues(Symbol, from = values_from, to = values_to))
  macro[[2]] <-  
    macro[[2]] %>% 
    rename_at(vars(values_from), ~ values_to)
  macro[[3]] <- 
    macro[[3]] %>% 
    rename_at(vars(values_from), ~ values_to)
  macro[[4]] <-
    macro[[4]] %>% 
    rename_at(vars(values_from), ~ values_to)
  
  release_dates <- 
    macro[[1]] %>%
    filter(Symbol == dep, realtime_period < as.Date("2021-01-31")) %>% 
    select(realtime_period)
  
  
  macro <<- macro
  
  survey <- ei_bssi %>%
    rename(esi   = `BS-ESI-I`,
           cci   = `BS-CCI-BAL`,
           csmci = `BS-CSMCI-BAL`,
           ici   = `BS-ICI-BAL`,
           rci   = `BS-RCI-BAL`,
           sci   = `BS-SCI-BAL`,
           realtime_period = date) %>%
    mutate(realtime_period = realtime_period + days(21)) %>% # add 21 days to approx real-time release date
    filter(country == country.i) %>%
    select(-country)
  
  # create lagged variables for mixed frequency
  
  lag_names     <- paste(formatC(1:3, width = nchar(3), flag = "0"), sep = "_")
  lag_functions <- setNames(paste("dplyr::lag(., ", 1:3, ")"), lag_names)
  
  survey <- 
    survey %>%
    mutate_at(vars(cci:sci), 
              funs_(lag_functions))
  
  # create the realtime daily matrix   ---------------------
  Xd              <- data.frame(realtime_period = macro[[4]]$realtime_period)
  survey.realtime <- left_join(Xd, survey, by="realtime_period")
  survey.realtime[survey.realtime == 0] <- NA
  survey.realtime <- survey.realtime %>%
    mutate_at(vars(-c(realtime_period)), 
              funs(rel = as.character(cumsum(as.numeric(!is.na(.))))))
  
  
  f_temp <- function(df.temp) {
    N    <- sum(grepl("_rel", names(df.temp)))
    gvar <- names(df.temp)[grep("_rel", names(df.temp))]
    
    for (i in 1:N)
    {
      gvar_i <- gvar[i] 
      mvar_i <- substr(gvar_i, 1, nchar(gvar_i)-4)
      
      df.temp = 
        df.temp %>%
        group_by(!!ensym(gvar_i)) %>%
        mutate(!!mvar_i  := first(!!ensym(mvar_i))) %>%
        ungroup 
    }
    
    df.temp <- select(df.temp, -contains("_rel"))
    return(df.temp)
  }
  
  survey.realtime <- f_temp(survey.realtime)
  
  #-----------------------------------------------------------
  
  if (mix.freq == TRUE) df.realtime    = plyr::join_all(list(select(macro[[4]], 
                                                                    realtime_period, 
                                                                    GDP, 
                                                                    starts_with("INDPRO"), 
                                                                    starts_with("CPI"), starts_with("UNEM")),
                                                             df.sentiment,
                                                             survey.realtime), 
                                                        by='realtime_period', 
                                                        type='left')
  
  if (mix.freq == FALSE)
  {
    macro[[5]]  <- select(macro[[4]], realtime_period, !contains("_")) %>%
      left_join(select(macro[[4]], realtime_period, contains("NFCI_")), by="realtime_period")
    
    df.realtime = plyr::join_all(list(macro[[5]], 
                                      df.sentiment, 
                                      select(survey.realtime, realtime_period, !contains("_"))),
                                 by='realtime_period', 
                                 type='left')
  }
  
  release_dates <<- 
    select(filter(macro[[1]], Symbol == dep, realtime_period < as.Date(final)), realtime_period)
  
  
  ## Save data for in-sample analysis
  insample_data_list <- list(df.realtime, release_dates)
  save(insample_data_list, file=paste0(store.dir, "insample_data_", country.i, ".RData"))
}


######################################################################
######################################################################

function_results <- function(country.i, 
                             mix.freq, 
                             type = "av", 
                             final = "2019-12-31", 
                             fun = "mean",
                             mymu_fluct = 10,
                             symb.list_all,
                             save.results, 
                             store.dir,
                             p.val = 0.1, ## critical value for the in-sample results
                             mt.correction = FALSE, ## IF TRUE, correct for multiple-testing with Adaptive BH
                             load_results = FALSE ## If TRUE, load already save in-sample and out-of-sample estimates
)
{
  
  if (type == "av")
  {
    sentiment <- DNA_sentiment_av %>%
      filter(country == country.i, measure == type) %>%
      select(realtime_period, topic, value) %>%
      spread(topic, value) %>%
      mutate(average = (economy+finsector+indp+inflation+monpol+unemployment)/6)
  }
  if (type == "oss")
  {
    sentiment <- DNA_sentiment_oss %>%
      filter(country == country.i, measure == "av") %>%
      select(realtime_period, topic, value) %>%
      spread(topic, value) %>%
      mutate(average = (economy+finsector+indp+inflation+monpol+unemployment)/6)
  }
  if (type == "cum")
  {
    sentiment <- DNA_sentiment_cum %>%
      filter(country == country.i, measure == type) %>%
      select(realtime_period, topic, value) %>%
      spread(topic, value) %>%
      mutate(average = (economy+finsector+indp+inflation+monpol+unemployment)/6)
  }
  
  sent.names <- names(sentiment)[-1]
  
  ############################################################################
  
  df.sentiment <- roll_sentiment_US(sentiment, roll_type,  lambda, roll.window.sent1, fun) 
  
  macro <- get_macrodata(country    = country.i, 
                         where      = "alfred", 
                         start.date = start.date, 
                         end.date   = end.date, 
                         vintage    = vintage,
                         save.it    = save.it, 
                         folder.fred = macro.dir, 
                         symb.list_all) 
  
  values_from   <- filter(symb.list_all, country == country.i) %>% select(symb) %>% unlist %>% c()
  values_to     <- filter(symb.list_all, country == country.i) %>% select(Description) %>% unlist %>% c()
  
  myvar     <- c("IND", "CPI","UNE")
  index       <- which(substr(values_to, 1, 3) %in% myvar)
  for (i in 1:3) values_from   <- c(values_from, paste0(values_from[index], "_",i))
  for (i in 1:3) values_to     <- c(values_to, paste0(values_to[index], "_",i))
  
  macro[[1]] <- 
    macro[[1]] %>% 
    mutate(Symbol = plyr::mapvalues(Symbol, from = values_from, to = values_to))
  macro[[2]] <-  
    macro[[2]] %>% 
    rename_at(vars(values_from), ~ values_to)
  macro[[3]] <- 
    macro[[3]] %>% 
    rename_at(vars(values_from), ~ values_to)
  macro[[4]] <-
    macro[[4]] %>% 
    rename_at(vars(values_from), ~ values_to)
  
  release_dates <- 
    macro[[1]] %>%
    filter(Symbol == dep, realtime_period < as.Date("2021-01-31")) %>% 
    select(realtime_period)
  
  
  macro <<- macro
  
  survey <- ei_bssi %>%
    rename(esi   = `BS-ESI-I`,
           cci   = `BS-CCI-BAL`,
           csmci = `BS-CSMCI-BAL`,
           ici   = `BS-ICI-BAL`,
           rci   = `BS-RCI-BAL`,
           sci   = `BS-SCI-BAL`,
           realtime_period = date) %>%
    mutate(realtime_period = realtime_period + days(21)) %>% # add 21 days to approx real-time release date
    filter(country == country.i) %>%
    select(-country)
  
  # create lagged variables for mixed frequency
  
  lag_names     <- paste(formatC(1:3, width = nchar(3), flag = "0"), sep = "_")
  lag_functions <- setNames(paste("dplyr::lag(., ", 1:3, ")"), lag_names)
  
  survey <- 
    survey %>%
    mutate_at(vars(cci:sci), 
              funs_(lag_functions))
  
  # create the realtime daily matrix   ---------------------
  Xd              <- data.frame(realtime_period = macro[[4]]$realtime_period)
  survey.realtime <- left_join(Xd, survey, by="realtime_period")
  survey.realtime[survey.realtime == 0] <- NA
  survey.realtime <- survey.realtime %>%
    mutate_at(vars(-c(realtime_period)), 
              funs(rel = as.character(cumsum(as.numeric(!is.na(.))))))
  
  
  f_temp <- function(df.temp) {
    N    <- sum(grepl("_rel", names(df.temp)))
    gvar <- names(df.temp)[grep("_rel", names(df.temp))]
    
    for (i in 1:N)
    {
      gvar_i <- gvar[i] 
      mvar_i <- substr(gvar_i, 1, nchar(gvar_i)-4)
      
      df.temp = 
        df.temp %>%
        group_by(!!ensym(gvar_i)) %>%
        mutate(!!mvar_i  := first(!!ensym(mvar_i))) %>%
        ungroup 
    }
    
    df.temp <- select(df.temp, -contains("_rel"))
    return(df.temp)
  }
  
  survey.realtime <- f_temp(survey.realtime)
  
  #-----------------------------------------------------------
  
  if (mix.freq == TRUE) df.realtime    = plyr::join_all(list(select(macro[[4]], 
                                                                    realtime_period, 
                                                                    GDP, 
                                                                    starts_with("INDPRO"), 
                                                                    starts_with("CPI"), starts_with("UNEM")),
                                                             df.sentiment,
                                                             survey.realtime), 
                                                        by='realtime_period', 
                                                        type='left')
  
  if (mix.freq == FALSE)
  {
    macro[[5]]  <- select(macro[[4]], realtime_period, !contains("_")) %>%
      left_join(select(macro[[4]], realtime_period, contains("NFCI_")), by="realtime_period")
    
    df.realtime = plyr::join_all(list(macro[[5]], 
                                      df.sentiment, 
                                      select(survey.realtime, realtime_period, !contains("_"))),
                                 by='realtime_period', 
                                 type='left')
  }
  
  release_dates <<- 
    select(filter(macro[[1]], Symbol == dep, realtime_period < as.Date(final)), realtime_period)
  
  

  if (load_results){
    load(file=paste0(store.dir, "in.gdp_", country.i, ".Rdata"))
  } else {
    in.gdp  <- doublelasso_analysis(dep       = dep, 
                                    indep     = indep, 
                                    sent      = sentx,
                                    mydata    = df.realtime,
                                    lag       = lag.max.q, 
                                    release_dates = release_dates, 
                                    ar.max    = ar.max.q, 
                                    x.max     = x.max, 
                                    nvar      = nvar.q,
                                    ncores    = ncores,
                                    mix.freq  = mix.freq,
                                    symb.list_all)
    
    if (save.results){save(in.gdp, file=paste0(store.dir, "in.gdp_", country.i, ".Rdata"))}
  }
  
  
  indep_from       <- c(sapply(indep, function(x) c(x,paste0(x,"_", 1:7))))
  indep_to         <-  purrr::map_chr(strsplit(indep_from, "_"),1)
  
  sel.gdp <- 
    in.gdp %>%
    mutate(p.value = ifelse(p.value <= p.val, p.value, NA)) %>%
    mutate(Sentiment = factor(toupper(Sentiment), levels = toupper(sentx))) %>%
    ggplot(aes(horizon, Sentiment)) + 
    geom_tile(fill = "white") + 
    geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf), 
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey97",  inherit.aes = FALSE)  +
    geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey85",  linetype = "dashed") + 
    geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85") + 
    geom_tile(data = pick(~ p.value <= p.val), aes(fill = p.value), colour = "white") + 
    #scale_fill_gradient2(low = "gray70", mid = "dodgerblue3", high = "tomato3", midpoint = 0.05)  +
    scale_fill_gradient(low = "dodgerblue4", high = "slategray2") +
    theme_bw(base_size = 12) + 
    labs(x = "", y = "", title = toupper(dep)) +
    #coord_cartesian(xlim = c(-20, max(lag.max.q)+20),  expand = FALSE) +
    scale_x_reverse(breaks = c(0,seq(75, max(lag.max.q),by=90)), labels = c("release", "t", "t-1", "t-2","t-3","t-4")) + 
    #scale_x_reverse() +
    scale_y_discrete(expand = c(0, 0)) + 
    theme(legend.position = "bottom", 
          axis.ticks = element_blank(), 
          axis.text.x = element_text(size = 13, angle = 0, hjust = 0.7),
          panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + 
    guides(fill = guide_colourbar(barwidth = 13, barheight = 1))
  
  ## Multiple testing correction
  if (mt.correction) {
    in.gdp_backup <- in.gdp
    in.gdp <- in.gdp_backup %>%
      group_by(Sentiment) %>%
      mutate(p.value.adaptBH = mutoss::adaptiveBH(p.value, alpha = 0.10, silent = TRUE)$adjPValues) %>%
      ungroup %>%
      mutate(p.value.adaptBH.pool = mutoss::adaptiveBH(p.value, alpha = 0.10, silent = TRUE)$adjPValues)
    #mutate(p.value = multtest::mt.rawp2adjp(p.value, proc="ABH"))
    

    sentxx <- sentx
    sentxx[sentxx == "indp"] <- "manuf"
    
    sel.gdp <-
      in.gdp %>%
      mutate(Sentiment = as.character(Sentiment)) %>%
      mutate(p.value = ifelse(p.value <= p.val, p.value.adaptBH, NA)) %>%
      mutate(Sentiment = ifelse(Sentiment == "indp", "MANUF", Sentiment)) %>%
      mutate(Sentiment = factor(toupper(Sentiment), levels = toupper(sentxx))) %>%
      ggplot(aes(horizon, Sentiment)) +
      geom_tile(fill = "white") +
      geom_rect(data=data.frame(xmin = 45, xmax = 135, ymin=-Inf, ymax=Inf),
                aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
                fill="grey85",  inherit.aes = FALSE)  +
      geom_vline(xintercept = c(0,seq(45, max(lag.max.q), by = 30)), color = "grey95",  linetype = "dashed") +
      geom_vline(xintercept = seq(45, max(lag.max.q), by = 90), color = "grey85") +
      geom_tile(data = pick(~ p.value <= p.val), aes(fill = p.value), colour = "white") +
      #scale_fill_gradient2(low = "gray70", mid = "dodgerblue3", high = "tomato3", midpoint = 0.05)  +
      scale_fill_gradient(low = "dodgerblue4", high = "slategray2") +
      theme_bw(base_size = 20) +
      labs(x = "", y = "", title = "GDP") +
      #coord_cartesian(xlim = c(-20, max(lag.max.q)+20),  expand = FALSE) +
      scale_x_reverse(breaks = c(0,seq(75, max(lag.max.q),by=90)), labels = c("rel.", "t", "t-1", "t-2","t-3","t-4")) +
      #scale_x_reverse() +
      scale_y_discrete(expand = c(0, 0), limits=rev) +
      theme(legend.position = "bottom",
            axis.ticks = element_blank(),
            axis.text.x = element_text(size = 18, angle = 0, hjust = 0.7),
            panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
      guides(fill = guide_colourbar(barwidth = 25, barheight = 1))+
      geom_hline(yintercept = 6.5, size = 0.5, alpha = 0.8)  
    
  }
  
  beta.gdp <- 
    in.gdp %>%
    mutate(Sentiment = factor(toupper(Sentiment), levels = toupper(sentx))) %>%
    ggplot(aes(horizon, Estimate)) + 
    geom_rect(data=data.frame(xmin = 30, xmax = 120, ymin=-Inf, ymax=Inf), 
              aes(xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax),
              fill="grey85",  inherit.aes = FALSE)  +
    geom_vline(xintercept = seq(0, max(lag.max.q), by = 30), color = "grey95",  linetype = "dashed") + 
    geom_vline(xintercept = seq(30, max(lag.max.q), by = 90), color = "grey85") + 
    geom_line(color = "dodgerblue4", size = 0.8)  +
    geom_ribbon(aes(ymin=(Estimate - 1.64 * Std..Error),ymax=(Estimate + 1.64 * Std..Error)), alpha=0.3, fill = "dodgerblue2")+
    geom_hline(yintercept = 0, linetype = "dashed") + 
    theme_bw(base_size = 15) + 
    labs(x = "",y = "", title = toupper(dep)) +
    #coord_cartesian(xlim = c(-20, max(lag.max.q)+20),  expand = FALSE) +
    scale_x_reverse(breaks = c(0,seq(75, max(lag.max.q),by=90)), 
                    labels = c("rel.", "t", "t-1", "t-2","t-3","t-4")) + 
    #scale_x_reverse() +
    #scale_y_discrete(expand = c(0, 0)) + 
    theme(legend.position = "none", 
          axis.ticks = element_blank(), 
          axis.text.x = element_text(size = 13, angle = 0, hjust = 0.7),
          panel.grid.major = element_blank(), panel.grid.minor = element_blank()) + 
    facet_wrap(~ Sentiment, ncol = 3, scales = "free")
  
  in.gdp.plots <- list(sel.gdp, beta.gdp)  
  
  # if (plot.save) ggsave(plot = beta.gdp, 
  #                       filename = paste0(plot.dir, "double_lasso_localprojection_GDP.pdf"),
  #                       width = 10, height = 6)
  
  

  
  # survey and sentiment together
  
  if (load_results){
    load(file=paste0(store.dir, "out.gdp.all_", country.i, ".Rdata"))
  } else {
    out.gdp.all   <- out_sample_analysis_R1(dep,
                                            indep,
                                            sentx,
                                            mydata = df.realtime,
                                            start.forecast,
                                            lag    = lag.max.q,
                                            ar.max = ar.max.q,
                                            x.max  = x.max,
                                            scheme = scheme,
                                            nvar = nvar.q,
                                            ncores = ncores,
                                            lasso.method = lasso.method,
                                            ar.selection = ar.selection,
                                            force.in = force.in,
                                            max.var = max.var,
                                            crossval = crossval,
                                            mix.freq   = TRUE,
                                            release_dates,
                                            symb.list_all)
    
    if (save.results){save(out.gdp.all, file=paste0(store.dir, "out.gdp.all_", country.i, ".Rdata"))}
  }
  
  
  
  out.gdp.all$forecast$average_arxn    = apply(select(out.gdp.all$forecast, contains("ARXN_")), 1,
                                               mean, na.rm=T)
  out.gdp.all$forecast$average_survey  = apply(select(out.gdp.all$forecast, starts_with("ARXN") & ends_with(sentx2)), 1, 
                                               mean, na.rm=T)
  out.gdp.all$forecast$average_sent    = apply(select(out.gdp.all$forecast, starts_with("ARXN") & ends_with(sentx1)), 1, 
                                               mean, na.rm=T)
  
  
  out.gdp.all.plots <- make_plots_outsample(res.gdp=out.gdp.all, 
                                            dep, lag.max.q, lagDM=lagDMq, freq = "q", start.forecast, 
                                            end.date, fluctuation = do.fluct, my_mu = mymu_fluct, 
                                            country.i=country.i)
  return(list(in.gdp.plots, out.gdp.all.plots, all = out.gdp.all$forecast,  insample = in.gdp))
}


doublelasso_analysis <- function(dep, indep, sent,  mydata = data, lag, release_dates, ar.max = 3, x.max = 3, 
                                 nvar = 5, ncores = 10, mix.freq = TRUE, symb.list_all = symb.list_all)
{
  # FUNCTION TO ESTIMATE THE MODELS IN-SAMPLE
  #
  # subset.method    = c("leaps","ss", "l0","lasso")
  # ss.method = c("adjr2", "bic")
  #
  
  dep1        <- "Y"
  n.horizon   <- length(lag)
  
  myf <- function(i, dep, dep1, indep, sent, mydata, release_dates, ar.max, 
                  x.max,  nvar,  mix.freq, symb.list_all)
  {
    model1  <- define_model(dep, indep, sent, ar.max, x.max, mix.freq, symb.list_all1 = symb.list_all)
    
    Y = mydata %>%
      filter(realtime_period %in% release_dates$realtime_period) %>%
      select(realtime_period, dep) %>%
      rename(Y = dep) %>%
      ungroup
    
    X = mydata %>%
      filter(realtime_period %in% (release_dates$realtime_period - i)) %>%
      mutate(realtime_period = realtime_period + i) %>%
      ungroup
    
    reg.data <- inner_join(X, Y, by = "realtime_period")
    
    # make variables used in the regression model to start at the same time
    # if (mix.freq)  var.sel <-  c("realtime_period", "Y", dep, indep, c(sapply(indep, function(x) paste0(x, "_", 1:3))), sent)
    # if (!mix.freq) var.sel <-  c("realtime_period", "Y", dep, indep, sent)
    # estimation <- select(reg.data, var.sel)
    
    # make variables used in the regression model to start at the same time
    if (mix.freq)  var.sel <-  c("realtime_period", "Y", dep, indep, c(sapply(indep, function(x) paste0(x, "_", 1:7))), sent)
    if (!mix.freq) var.sel <-  c("realtime_period", "Y", dep, indep, sent)
    var.sel    <- unique(var.sel)
    estimation <- select(reg.data, which(names(reg.data) %in% var.sel))
    
    index       <- which(apply(estimation, 1, function(x) sum(is.na(x))) > 0)
    if (length(index) > 0) estimation <- estimation[-(index),]
    
    # REGRESSIONS   ---------------------------------------------------------------------------------------------------
    
    fit.ds  <- doublelasso_arxn_model(dep1, dep, indep, sent, estimation, ar.max,
                                      x.max, nvar, mix.freq, symb.list_all)
    
    mat.ds <- data.frame(horizon = i, 
                         Sentiment = sent,
                         Estimate = fit.ds$coefficients,
                         `Std. Error` = fit.ds$se,
                         `t value` = fit.ds$t,
                         `p-value` = fit.ds$pval)
    #  ---------------------------------------------------------------------------------------------------
    
    return(mat.ds)
  }
  
  t0 = Sys.time()
  
  use_core <- min(n.horizon, ncores)
  cluster = makeCluster(use_core, type = "SOCK")
  registerDoSNOW(cluster)
  
  pb       <- txtProgressBar(max = n.horizon, style = 3)
  progress <- function(n) setTxtProgressBar(pb, n)
  opts     <- list(progress = progress)
  
  val = foreach(i =1:n.horizon, 
                .combine = 'rbind',
                .packages = c("dplyr", "formula.tools", "HDeconometrics", "sandwich"),
                .verbose  = FALSE,
                .export   = c("define_model", "doublelasso_arxn_model"),
                .options.snow = opts) %dopar% 
    myf(lag[i], dep, dep1, indep, sent, mydata, release_dates, ar.max, 
        x.max, nvar, mix.freq, symb.list_all)
  
  close(pb)
  stopCluster(cluster)
  
  t1 = Sys.time() - t0 
  
  return(val)
}

doublelasso_arxn_model <- function(dep1, dep, indep, sent,  data, ar.max = 6, 
                                   x.max = 6, nvar = 8,  mix.freq = TRUE, symb.list_all = symb.list_all)
{
  # Double LASSO (implemented in package hdm)
  
  library(hdm)
  
  model1       <- define_model(dep, indep, sent, ar.max, x.max, mix.freq, symb.list_all1 = symb.list_all)
  if (is.null(ar.max))  model.subset <- as.formula(paste0(dep1," ~ ", paste0(paste0(model1[3],"+", model1[4]), collapse = "+"))) 
  if (!is.null(ar.max)) model.subset <- as.formula(paste0(dep1," ~ ", paste0(model1[1], collapse = "+")))
  fit.mat      <- model.frame(model.subset, data) %>% as.data.frame
  
  nvar <- ncol(fit.mat)
  N    <- nrow(fit.mat) 
  
  # fit.po <- rlassoEffects(x     = fit.mat[,-1],
  #                         y     = fit.mat[,1],
  #                         index = (nvar-length(sent)):(nvar-1),
  #                         method = "partialling out")
  
  fit.ds <- try(
    rlassoEffects(x      = fit.mat[,-1], 
                  y      = fit.mat[,1],
                  index  = (nvar-length(sent)):(nvar-1), 
                  method = "double selection"),
    silent = TRUE)
  if (class(fit.ds) == "try-error")
  {
    fit.ds <- rlassoEffects(x     = fit.mat[,-1],
                            y     = fit.mat[,1],
                            index = (nvar-length(sent)):(nvar-1),
                            method = "partialling out")
  }
  
  return(fit.ds)
}

define_model <- function(dep, indep, sent = NULL, ar.max = 6, x.max = 0,
                         mix.freq = TRUE, symb.list_all1 = symb.list_all)
{
  # dep = CFNAI, CPIAUCSL, GDPC1, ICSA, INDPRO, PAYEMS, UNRATE
  
  if (is.null(ar.max)) dep.lag = "1" else dep.lag  = paste0("lag(", dep, ",", 0:ar.max,")", collapse="+")
  if (length(indep) > 0)
  {
    indep.freq <- symb.list_all1[symb.list_all1$Description %in% indep,]
    if (mix.freq)
    {
      indep.lag = indep
      for (i in 1:length(indep))
      {
        if (indep.freq$freq[i] == "m") indep.lag = c(indep.lag, paste0(indep.freq$Description[i],"_",1:3))
        if (indep.freq$freq[i] == "w") indep.lag = c(indep.lag, paste0(indep.freq$Description[i],"_",1:7))
      }
      indep.lag <- paste0(indep.lag, collapse = "+")
    }
    if (!mix.freq) indep.lag = paste0(indep, collapse="+")
  } else indep.lag = ""
  
  #sent.lag = paste0("lag(",sent, ",", 0:sent.max,")", collapse="+")
  if (length(sent) > 0) sent.lag = paste0("lag(",sent, ",", 0,")", collapse="+") else sent.lag = ""
  
  if (length(indep) == 0 & length(sent) == 0) all.var <- dep.lag
  if (length(indep) > 0 & length(sent) == 0)  all.var <- paste(dep.lag, indep.lag, sep = "+")
  if (length(indep) == 0 & length(sent) > 0)  all.var <- paste(dep.lag, sent.lag, sep = "+")
  if (length(indep) > 0 & length(sent) > 0)   all.var <- paste(dep.lag, indep.lag, sent.lag, sep = "+")
  
  out = list(all.var,
             dep.lag,
             indep.lag,
             sent.lag)
  
  return(out)
}


out_sample_analysis_R1 <- function(dep, indep, sent, mydata = data, start.forecast, lag, ar.max = 6, x.max = 1, scheme = "recursive",
                                   nvar = 5, ncores = 20, lasso.method = "cv", 
                                   ar.selection = "yes", force.in = 1, max.var = 20, 
                                   crossval = TRUE, mix.freq = FALSE, release_dates, 
                                   symb.list_all = symb.list_all, myc = 1.1)
{
  #
  # function to produce out-of-sample forecasts for the conditional mean
  #
  
  n.horizon      <- length(lag)
  #release_dates  <- filter(macro[[1]], Symbol == dep) %>% select(realtime_period)
  start.index    <- max(which(release_dates$realtime_period <= as.Date(start.forecast)))
  end.index      <- max(which(release_dates$realtime_period <= as.Date("2020-12-31")))
  
  #--------------------------------------------------------------------------------------------------------
  # function that produces the forecasts for all models at a given horizon
  
  myf <- function(horizon, mydata, dep, indep, sent, release_dates, start.index, end.index,  
                  scheme, ar.max, x.max, how, nvar, lasso.method, ss.method, ar.selection,
                  force.in, max.var, crossval, subset.method, symb.list_all, myc)
  {
    
    dep1         <- "Y"
    model1       <- define_model(dep, indep, sent, ar.max, x.max, mix.freq)
    store.forecast <- data.frame()
    store.ss       <- vector("list", end.index-start.index)
    store.arx      <- store.ss
    store.news     <- store.ss
    
    for (t in start.index:(end.index-1))
    {
      
      if (scheme == "recursive") target.date <- 1:(t+1)
      if (scheme == "rolling")   target.date <- (t-start.index+1):(t+1)
      
      Y.target =
        mydata %>% 
        filter(realtime_period %in% (release_dates$realtime_period[target.date])) %>% 
        select(realtime_period, dep) %>%
        rename(Y = dep) %>%
        ungroup
      
      X.target =
        mydata %>% 
        filter(realtime_period %in% (release_dates$realtime_period[target.date]- horizon)) %>% 
        mutate(realtime_period = realtime_period + horizon) %>% 
        ungroup
      
      # ............................................................................................................
      
      X.estimation <- X.target[-nrow(X.target),]
      Y.estimation <- Y.target[-nrow(Y.target),]
      
      estimation  <- inner_join(X.estimation, Y.estimation, by = 'realtime_period')
      target      <- inner_join(X.target, Y.target, by = "realtime_period")
      
      # make variables used in the regression model to start at the same time
      #      var.sel     <-  c("realtime_period", "Y", dep, indep, c(sapply(indep, function(x) paste0(x, "_", 1:3))), sent, 
      #                        "CFNAI", macro1, macro2)
      
      # make variables used in the regression model to start at the same time
      if (mix.freq)  var.sel <-  c("realtime_period", "Y", dep, indep, c(sapply(indep, function(x) paste0(x, "_", 1:7))), sent)
      if (!mix.freq) var.sel <-  c("realtime_period", "Y", dep, indep, sent)
      estimation <- select(estimation, which(names(estimation) %in% var.sel))
      
      index <- which(apply(estimation, 1, function(x) sum(is.na(x))) > 0)
      if (length(index) > 0) estimation <- estimation[-(index),]
      
      # REGRESSIONS   ---------------------------------------------------------------------------------------------------
      
      if (ar.selection == "yes")
      {
        model.ar  <- rlasso_arxn_model(dep1, dep, indep = NULL, sent = NULL, estimation, ar.max, x.max, nvar, alpha, crit = lasso.method, crossval, force.in, mix.freq, myc)
        model.arx <- rlasso_arxn_model(dep1, dep, indep, sent = NULL, estimation, ar.max, x.max, nvar, alpha, crit = lasso.method, crossval, force.in,mix.freq, myc)
      } else {
        model.ar  <- as.formula(paste0(dep1, " ~ ", paste0(model1[2], collapse = "+")))
        model.arx <- rlasso_arxn_model(dep1, dep, indep, sent = NULL, estimation, ar.max, x.max, nvar, alpha, crit = lasso.method, crossval, force.in, mix.freq, myc)
      }
      
      model.ss     <- rlasso_arxn_model(dep1, dep, indep, sent, estimation, ar.max, x.max, nvar, alpha, crit = lasso.method, crossval, force.in, mix.freq, myc)
      model.news   <- rlasso_arxn_model(dep1, dep, indep = NULL, sent, estimation, ar.max, x.max, nvar, alpha, crit=lasso.method, crossval, force.in, mix.freq, myc)
      
      #  ---------------------------------------------------------------------------------------------------
      
      fit.ar     <- lm(model.ar,   data = estimation)
      fit.arx    <- lm(model.arx,  data = estimation)
      fit.ss     <- lm(model.ss,   data = estimation)
      fit.news   <- lm(model.news, data = estimation)
      
      # store.ss[[t-start.index+1]]   <- summary(fit.ss)
      # store.arx[[t-start.index+1]]  <- summary(fit.arx)
      # store.news[[t-start.index+1]] <- summary(fit.news)
      
      store.ss[[t-start.index+1]]   <- fit.ss
      store.arx[[t-start.index+1]]  <- fit.arx
      store.news[[t-start.index+1]] <- fit.news
      
      forecast.ar   <- predict(fit.ar, newdata = target)  %>% tail(1) %>% c()
      names(forecast.ar)  <- "AR"
      forecast.arx  <- predict(fit.arx, newdata = target) %>% tail(1) %>% c()
      names(forecast.arx) <- "ARX"
      forecast.ss   <- predict(fit.ss, newdata = target)  %>% tail(1) %>% c()
      names(forecast.ss)  <- "SS"
      forecast.news   <- predict(fit.news, newdata = target)  %>% tail(1) %>% c()
      names(forecast.news)  <- "NEWS"
      
      temp_arn   <- c()
      temp_arxn  <- c()
      temp_arpcn <- c()
      
      for (j in 1:length(sent))
      {
        model.arn   <- as.formula(paste0(as.character(model.ar), "+ " , sent[j]))
        model.arxn  <- as.formula(paste0(as.character(model.arx), "+ " , sent[j]))
        
        fit.arn   <- lm(model.arn,  data = estimation)
        fit.arxn  <- lm(model.arxn, data = estimation)
        
        forecast.arn   <- predict(fit.arn, newdata = target)  %>% tail(1)
        forecast.arxn  <- predict(fit.arxn, newdata = target) %>% tail(1)
        
        names(forecast.arn)   <- paste0("ARN_",sent[j])
        names(forecast.arxn)  <- paste0("ARXN_",sent[j])
        
        temp_arn   <- c(temp_arn, forecast.arn)
        temp_arxn  <- c(temp_arxn, forecast.arxn)
      }
      
      realization      <- tail(target$Y,1)
      realization.date <- as.character(tail(target$realtime_period,1))
      forecast  <- c(forecast.ar, forecast.arx, forecast.ss, forecast.news, temp_arn, temp_arxn)
      results   <- data.frame(Y = realization,
                              realtime_period = realization.date, 
                              t(forecast),
                              horizon = horizon,
                              stringsAsFactors = FALSE)
      
      store.forecast   <- rbind(store.forecast, results)
    }
    
    return(list(store.forecast, store.ss, store.arx, store.news))
  }
  # end
  #-----------------------------------------------------------------------------------------------------------
  
  # #  serial implementation
  #  store.forecast <- vector("list", length(horizon.plot))
  #  for (i in 1:length(horizon.plot)) store.forecast[[i]] = myf(horizon.plot[i], mydata, dep, indep, sent, release_dates,
  #                                                              start.index, end.index, ar.max, x.max, how, nvar, lasso.method, ss.method)
  
  t0 = Sys.time()
  
  use_cores <- min(n.horizon, ncores)
  cluster = makeCluster(use_cores, type = "SOCK")
  registerDoSNOW(cluster)
  
  pb       <- txtProgressBar(max = n.horizon, style = 3)
  progress <- function(n) setTxtProgressBar(pb, n)
  opts     <- list(progress = progress)
  
  val = foreach(i = 1:n.horizon, 
                .combine = 'rbind',
                .packages = c("dplyr", "leaps", "glmnet", "formula.tools", 
                              "HDeconometrics", "L0Learn"),
                .verbose  = FALSE,
                .export   = c("define_model", "rlasso_arxn_model", "lasso_arxn_model"),
                .options.snow = opts) %dopar% 
    myf(lag[i], mydata, dep, indep, sent, release_dates, 
        start.index, end.index, scheme, ar.max, x.max, how, nvar,
        lasso.method, ss.method, ar.selection, force.in, 
        max.var, crossval, subset.method, symb.list_all, myc)
  
  close(pb)
  stopCluster(cluster)
  
  t1 = Sys.time() - t0  
  
  store.forecast   <- data.frame(do.call(rbind, val[,1]))
  store.forecast$realtime_period = as.Date(store.forecast$realtime_period)
  
  
  return(list(forecast = store.forecast, fit.ss = val[,2], 
              fit.arx = val[,3], fit.news = val[,4]))  
}


rlasso_arxn_model <- function(dep1, dep, indep, sent,  data, ar.max = 6, x.max = 6, nvar = 8, alpha = 1, 
                              crit = "cv", crossval = FALSE, force.in = NULL, mix.freq = TRUE, myc = 1.1)
{
  # Rigourous LASSO (implemented in package hdm)
  
  library(hdm)
  
  if (!is.null(ar.max))
  {
    n.reg <- ifelse(!is.null(sent),
                    1 + ar.max + 5 * length(indep) + length(sent),
                    1 + ar.max + 5 * length(indep)) 
  } else {
    n.reg <- ifelse(!is.null(sent),
                    1 + 5 * length(indep) + length(sent),
                    1 + 5 * length(indep)) 
  }
  
  model1       <- define_model(dep, indep, sent, ar.max, x.max, mix.freq)
  
  if (is.null(ar.max)) 
  {
    if (is.null(sent) & !is.null(indep)) model.subset = as.formula(paste0(dep1," ~ ", paste0(model1[3], collapse = "+"))) 
    if (is.null(sent) & is.null(indep))  model.subset = as.formula(paste0(dep1," ~ 1"))
    if (!is.null(sent) & !is.null(indep)) model.subset = as.formula(paste0(dep1," ~ ", paste0(paste0(model1[3],"+", model1[4]), collapse = "+"))) 
    if (!is.null(sent) & is.null(indep)) model.subset = as.formula(paste0(dep1," ~ ", paste0(model1[4], collapse = "+"))) 
  } else {
    if (is.null(sent) & !is.null(indep)) model.subset = as.formula(paste0(dep1," ~ ", paste0(paste0(model1[2],"+", model1[3]), collapse = "+"))) 
    if (is.null(sent) & is.null(indep))  model.subset = as.formula(paste0(dep1," ~ ", paste0(model1[2], collapse = "+")))
    if (!is.null(sent) & !is.null(indep)) model.subset = as.formula(paste0(dep1," ~ ", paste0(paste0(model1[2],"+", model1[3],"+", model1[4]), collapse = "+"))) 
    if (!is.null(sent) & is.null(indep)) model.subset = as.formula(paste0(dep1," ~ ", paste0(paste0(model1[2],"+", model1[4]), collapse = "+")))
  }
  
  fit.mat      <- model.frame(model.subset, data) %>% as.data.frame
  
  keep_x <- rep(1, ncol(fit.mat)-1)
  if (length(force.in) > 0) keep_x[force.in] <- 0
  
  nvar <- min(nvar, n.reg)
  N    <- nrow(fit.mat) 
  
  fit.lasso   <- rlasso(Y ~ ., 
                        data = fit.mat, 
                        post  =TRUE,
                        penalty = list(c = myc))
  
  sel.lasso   <- stringr::str_replace_all(names(which(fit.lasso$index)), "`","")
  if (length(sel.lasso) > 0)   model.lasso = as.formula(paste0(dep1," ~ ", paste0(sel.lasso, collapse = "+")))
  if (length(sel.lasso) == 0)  model.lasso = as.formula(paste0(dep1," ~ 1"))
  
  return(model.lasso)
}

#########################               PRINCIPAL COMPONENT functions


pc_sentiment <- function(sentiment.roll, nfact = 6)
{
  index <- which(apply(sentiment.roll, 1, function(x) sum(is.na(x))) > 0)
  temp  <- data.frame(sentiment.roll$realtime_period, 
                      matrix(NA, nrow(sentiment.roll), nfact)) 
  names(temp) <- c("realtime_period", paste0("sentpc",1:nfact))
  
  # --- PC of all sentiment indicators
  pc  <- prcomp(na.omit(sentiment.roll[-index,-1]), center = TRUE, scale = TRUE)
  #plot(pc, type = "lines")
  temp[-index,-1] <- pc$x[,1:nfact]
  
  return(temp)
}


pc_macro <- function(macro.df, start.forecast)
{
  var.monthly <- filter(symb.list, freq == "m") %>% select(symb) %>% c()
  macro.pc    <- select(macro.df, var.monthly$symb)
  
  index    <- which(apply(macro.df, 1, function(x) sum(is.na(x))) > 0)
  temp     <- data.frame(sentiment.roll$realtime_period, 
                         matrix(NA, nrow(sentiment.roll), 5)) 
  names(temp) <- c("realtime_period", paste0("sentpc",1:5))
  
  # --- PC of all sentiment indicators
  pc  <- prcomp(na.omit(sentiment.roll[-index,-1]), center = TRUE, scale = TRUE)
  
  temp[-index,-1] <- pc$x[,1:5]
  
  return(temp)
}



