## Replication file for "Forecasting GDP in Europe with textual data" (Barbaglia, Consoli, Manzan)
## https://papers.ssrn.com/sol3/papers.cfm?abstract_id=3898680
## The The Python code to perform to produce the sentiment indicators is available at \url{https://github.com/sergioconsoli/SentiBigNomics} and a {\tt R} package is available at \url{https://github.com/lucabarbaglia/FiGASR}
## The proposed sentiment indicators are availble at \url{https://github.com/lucabarbaglia/FiGASR}

# sessionInfo()
# R version 3.6.0 (2019-04-26)
# Platform: x86_64-redhat-linux-gnu (64-bit)
# Running under: CentOS Linux 7 (Core) 

rm(list=ls())

wd        <- "./JAE_replication_files/" ## location of this R script
setwd(wd)

plot.dir    <- "./Figures/"  
tab.dir     <- "./Tables/"   
store.dir   <- "./RData/"   
macro.dir   <-  "./RData/FRED_EU/"  
ncores      <- 60 ## number of cores

if (!dir.exists(plot.dir)){dir.create(plot.dir)}
if (!dir.exists(tab.dir)){dir.create(tab.dir)}

source("aux_functions.R")  ## manually install the packages that are missing

## Load Codes for MacroEconomic Variables
symb.list <- read_excel(paste0(store.dir, "FiGAS_EU_Alfred_codes.xlsx"), 
                        col_types = c("text", "text", "text", "text", "date", "date", "text")) %>%
  mutate(Description = gsub(": ", "\n", Description))

## PARAMATERS
countries     <- c("IT", "FR", "DE", "ES", "UK")
country_names <- c("Italy", "France", "Germany", "Spain", "United Kingdom")
tense         <- "PaPrFuNaN"
topics        <- c("economy", "unemployment", "monpol", "finsector", "inflation", "indp")
msr           <- "av" # av, cum or n
where         <- "alfred"
vintage       <- 1
save.it       <- FALSE 
save.results  <- F  

cepr.min <- c("2008-04-01", "2011-10-01","2020-01-01") 
cepr.max <- c("2009-06-01","2013-03-01","2020-07-01")
cepr.min <- as.Date(cepr.min)
cepr.max <- as.Date(cepr.max)


# create a daily time series with 1 if recession and 0 otherwise

nber <- seq(as.Date("1995/01/01"), as.Date("2020/12/31"), "days")
nber <- data.frame(date = nber, nber = 0)

nber <- nber %>%
  mutate(
    nber = case_when(
      date >= cepr.min[1] & date <= cepr.max[1] ~ "1",
      date >= cepr.min[2] & date <= cepr.max[2] ~ "1",
      date >= cepr.min[3] & date <= cepr.max[3] ~ "1"),
    nber = ifelse(is.na(nber), 0 ,1))

############################
#  SENTIMENT
############################


load(paste0(store.dir, "DNA_sentiment_smooth_av.RData"))
DNA_sentiment_av  <- filter(DNA_sentiment_smooth)
load(paste0(store.dir, "DNA_sentiment_smooth_oss_av.RData"))
DNA_sentiment_oss  <- filter(DNA_sentiment_oss_smooth)
load(paste0(store.dir, "DNA_sentiment_smooth_cum.RData"))
DNA_sentiment_cum  <- filter(DNA_sentiment_smooth)

##########################
# SURVEY
##########################
if (file.exists(paste0(store.dir, "ei_bssi.Rdata"))){
  load(paste0(store.dir, "ei_bssi.Rdata"))
} else {
  library(eurostat)
  ECFIN_raw <- get_eurostat("ei_bssi_m_r2") 
  
  ei_bssi <- ECFIN_raw %>%
    filter(geo %in% countries,
           s_adj == "SA") %>%
    mutate(country = as.character(geo)) %>%
    select(country, date=time, indic, values) %>%
    spread(indic, values)
  
  if(save.results){save(ei_bssi, file=paste0(store.dir, "ei_bssi.Rdata"))}
}


##############################
# IN-SAMPLE ANALYSIS
##############################

measure.i <- "av"
type      <- measure.i
vintage   <- 1
roll.window.sent1 <- 90         # rolling window for sentiment indicators
fun               <- "mean"     # "sum" or "mean"lambda         <- 0.10
lambda            <- 0.10
roll_type         <- "sma"      # "ema" (exponential MA) or  "sma" (simple MA)
table_type        <- "text"     # "text"  or "html" or "latex"

dep       <- "GDP" 
# indep     <- c("INDPRO","UNEM", "CPI")
# sentx     <- c("economy","finsector","indp","inflation","monpol",
#                "unemployment", "esi","ici","rci","sci","cci","csmci")
# sentx1    <- c("economy","finsector","indp","inflation","monpol",
#                "unemployment")
# sentx2    <- c("esi","ici","rci","sci","cci","csmci")
# type      <- measure.i
indep     <- c("UNEM","CPI", "INDPRO", "esi","ici","rci","sci","cci","csmci")
sentx     <- c("economy","finsector","indp","inflation","monpol", "unemployment")
sentx1    <- c("economy","finsector","indp","inflation","monpol","unemployment")
sentx2    <- c("economy","finsector","indp","inflation","monpol","unemployment")

mix.freq       <- TRUE
l0.penalty     <- "L0"                  # "L0", "L0L1", "L0L2"   
ss.how         <- "exhaustive"          # "exhaustive", "backward", "forward", "seqrep"
alpha          <- 1 
max.var        <- 20    
force.in       <- 1
crossval       <- FALSE
ss.method      <- "bic"                 # "adjr2" or "bic"
lasso.method   <- "cv"                  # "cv" or "bic" 
ar.selection   <- "no"                 # "yes" or "no"  
lag.max.q      <- seq(15, 500, by=15)
ar.max.q       <- 0                    # starts at 0 for lag 1
x.max          <- 0                    # starts at 0 for lag 1
lagDMq          <- 3
nvar.q         <- 5
subset.method  <- "lasso" 
plot.save      <- TRUE

benchmark1       <- paste0("Y ~ economy + unemployment")
benchmark2       <- paste0("Y ~ esi + csmci")
start.date       <- "1997-01-01"
end.date         <- "2020-01-01" #"2020-12-01"
start.forecast   <- "2007-01-01"
final            <- "2019-12-31"
scheme           <- "recursive"             # "recursive" or "rolling"
do.fluct         <- TRUE
my_mu            <- 20
myc              <- 0.8


###################################
### IN-SAMPLE RESULTS FOR FIG 4 ###
###################################

dep       <- "GDP" 
indep     <- c("INDPRO","UNEM", "CPI")
sentx     <- c("economy","finsector","indp","inflation","monpol",
               "unemployment", "esi","ici","rci","sci","cci","csmci")
sentx1    <- c("economy","finsector","indp","inflation","monpol",
               "unemployment")
sentx2    <- c("esi","ici","rci","sci","cci","csmci")
# type      <- measure.i
# indep     <- c("UNEM","CPI", "INDPRO", "esi","ici","rci","sci","cci","csmci")
# sentx     <- c("economy","finsector","indp","inflation","monpol", "unemployment")
# sentx1    <- c("economy","finsector","indp","inflation","monpol","unemployment")
# sentx2    <- c("economy","finsector","indp","inflation","monpol","unemployment")


mt_res.es <- function_results(country.i = "ES", mix.freq = TRUE, type, final, fun, mymu_fluct=my_mu, 
                              symb.list_all=symb.list, save.results=F, 
                              store.dir=store.dir, mt.correction = TRUE, p.val=0.05, load_results=F)
mt_res.fr <- function_results(country.i = "FR", mix.freq = TRUE, type, final, fun, mymu_fluct=my_mu, 
                              symb.list_all=symb.list, save.results=save.results, 
                              store.dir=store.dir, mt.correction = TRUE, p.val=0.05, load_results=F)
mt_res.it <- function_results(country.i = "IT", mix.freq = TRUE, type, final, fun, mymu_fluct=my_mu, 
                              symb.list_all=symb.list, save.results=save.results, 
                              store.dir=store.dir, mt.correction = TRUE, p.val=0.05, load_results=F)
mt_res.de <- function_results(country.i = "DE", mix.freq = TRUE, type, final, fun, mymu_fluct=my_mu, 
                              symb.list_all=symb.list, save.results=save.results, 
                              store.dir=store.dir, mt.correction = TRUE, p.val=0.05, load_results=F)
mt_res.uk <- function_results(country.i = "UK", mix.freq = TRUE, type, final, fun, mymu_fluct=my_mu, 
                              symb.list_all=symb.list, save.results=save.results, 
                              store.dir=store.dir, mt.correction = TRUE, p.val=0.05, load_results=F)

insample.sel.plot <- ggpubr::ggarrange(mt_res.de[[1]][[1]] + ggtitle("DE") , 
                                       mt_res.es[[1]][[1]] + ggtitle("ES") ,
                                       mt_res.fr[[1]][[1]] + ggtitle("FR") ,
                                       mt_res.it[[1]][[1]] + ggtitle("IT") ,
                                       mt_res.uk[[1]][[1]] + ggtitle("UK") ,
                                       ncol = 2, nrow = 3, common.legend = TRUE, legend = "bottom")

if (plot.save) ggsave(plot   = insample.sel.plot,
                      filename = paste0(plot.dir, paste0(toupper(dep),"_figasEU_insample_doublelasso_significance_MT_pval05.pdf")),
                      width  = 14,
                      height = 12)




##########################################
### IN-SAMPLE REULTS FOR OTHER FIGURES ###
##########################################


dep       <- "GDP" 
# indep     <- c("INDPRO","UNEM", "CPI")
# sentx     <- c("economy","finsector","indp","inflation","monpol",
#                "unemployment", "esi","ici","rci","sci","cci","csmci")
# sentx1    <- c("economy","finsector","indp","inflation","monpol",
#                "unemployment")
# sentx2    <- c("esi","ici","rci","sci","cci","csmci")
# type      <- measure.i
indep     <- c("UNEM","CPI", "INDPRO", "esi","ici","rci","sci","cci","csmci")
sentx     <- c("economy","finsector","indp","inflation","monpol", "unemployment")
sentx1    <- c("economy","finsector","indp","inflation","monpol","unemployment")
sentx2    <- c("economy","finsector","indp","inflation","monpol","unemployment")


### Multiple Testing Correction
mt_res.es <- function_results(country.i = "ES", mix.freq = TRUE, type, final, fun, mymu_fluct=my_mu, 
                              symb.list_all=symb.list, save.results=save.results, 
                              store.dir=store.dir, mt.correction = TRUE, p.val=0.10, load_results=F)
mt_res.fr <- function_results(country.i = "FR", mix.freq = TRUE, type, final, fun, mymu_fluct=my_mu, 
                              symb.list_all=symb.list, save.results=save.results, 
                              store.dir=store.dir, mt.correction = TRUE, p.val=0.10, load_results=F)
mt_res.it <- function_results(country.i = "IT", mix.freq = TRUE, type, final, fun, mymu_fluct=my_mu, 
                              symb.list_all=symb.list, save.results=save.results, 
                              store.dir=store.dir, mt.correction = TRUE, p.val=0.10, load_results=F)
mt_res.de <- function_results(country.i = "DE", mix.freq = TRUE, type, final, fun, mymu_fluct=my_mu, 
                              symb.list_all=symb.list, save.results=save.results, 
                              store.dir=store.dir, mt.correction = TRUE, p.val=0.10, load_results=F)
mt_res.uk <- function_results(country.i = "UK", mix.freq = TRUE, type, final, fun, mymu_fluct=my_mu, 
                              symb.list_all=symb.list, save.results=save.results, 
                              store.dir=store.dir, mt.correction = TRUE, p.val=0.10, load_results=F)

insample.sel.plot <- ggpubr::ggarrange(mt_res.de[[1]][[1]] + ggtitle("DE") , 
                                       mt_res.es[[1]][[1]] + ggtitle("ES") ,
                                       mt_res.fr[[1]][[1]] + ggtitle("FR") ,
                                       mt_res.it[[1]][[1]] + ggtitle("IT") ,
                                       mt_res.uk[[1]][[1]] + ggtitle("UK") ,
                                       ncol = 2, nrow = 3, common.legend = TRUE, legend = "bottom")


## Fluctuation
outsample.fluct <- ggpubr::ggarrange(mt_res.de[[2]][[3]] + ggtitle("DE") + facet_wrap(~ horizon, nrow = 1),
                                     mt_res.es[[2]][[3]] + ggtitle("ES") + facet_wrap(~ horizon, nrow = 1),
                                     mt_res.fr[[2]][[3]] + ggtitle("FR") + facet_wrap(~ horizon, nrow = 1),
                                     mt_res.it[[2]][[3]] + ggtitle("IT") + facet_wrap(~ horizon, nrow = 1),
                                     mt_res.uk[[2]][[3]] + ggtitle("UK") + facet_wrap(~ horizon, nrow = 1),
                                     ncol = 1, nrow = 5,
                                     common.legend = TRUE, legend = "bottom")

if (plot.save) ggsave(plot   = outsample.fluct,
                      filename = paste0(plot.dir, paste0(toupper(dep),"_figasEU_outsample_fluctuation_conditioning1_SM1.pdf")),
                      width  = 14,
                      height = 16)


## MSFE
outsample.msfe.rel <- ggpubr::ggarrange(mt_res.de[[2]][[7]] + ggtitle("DE"),
                                        mt_res.es[[2]][[7]] + ggtitle("ES"),
                                        mt_res.fr[[2]][[7]] + ggtitle("FR"),
                                        mt_res.it[[2]][[7]] + ggtitle("IT"),
                                        mt_res.uk[[2]][[7]] + ggtitle("UK"),
                                        ncol = 2, nrow = 3,
                                        common.legend = TRUE, legend = "bottom")

if (plot.save) ggsave(plot   = outsample.msfe.rel,
                      filename = paste0(plot.dir, paste0(toupper(dep),"_figasEU_outsample_MSFE_REL_SM1.pdf")),
                      width  = 14,
                      height = 16)








